#############
# $Id: bam_alignment.pm 15858 2012-07-10 15:36:19Z mg8 $
# Created By: gq1
# Last Maintained By: $Author: mg8 $
# Created On: 2011-10-19
# Last Changed On: $Date: 2012-07-10 16:36:19 +0100 (Tue, 10 Jul 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/archive/file/generation/bam_alignment.pm $

package npg_pipeline::archive::file::generation::bam_alignment;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;

extends q{npg_pipeline::archive::file::generation};

with qw{
  npg_common::roles::run::lane::tag_info
};

#scripts/bam_alignment.pl --input  input.bam --output_prefix output --id_run 6551 --position 1 --tag_index

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 15858 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $BAM_ALIGNMENT_SCRIPT  => q{/software/solexa/bin/bam_alignment.pl};

sub generate {
  my ( $self, $arg_refs ) = @_;

  my $id_run = $self->id_run();

  $self->log( qq{Creating Jobs to create bam files for $id_run} );

  my @positions = $self->positions($arg_refs);

  my $href = $self->lanes_by_split_nonsplit( @positions );

  my @bsub_commands;

  my $indexed_lanes     = $href->{indexed_lanes};
  if( scalar @{$indexed_lanes} ){

     push @bsub_commands, $self->_generate_bsub_commands({positions => $indexed_lanes,
                                                          multiplex   => 1,
                                                          arg_refs  =>$arg_refs});
  }

  my $non_indexed_lanes = $href->{non_indexed_lanes};
  if( scalar @{$non_indexed_lanes} ) {

     push @bsub_commands, $self->_generate_bsub_commands({positions => $non_indexed_lanes,
                                                         multiplex    => 0,
                                                         arg_refs =>$arg_refs});
  }


  my @job_ids;

  foreach my $cmd ( @bsub_commands ) {
    push @job_ids, $self->submit_bsub_command( $cmd );
  }

  return @job_ids;
}

###############
# responsible for generating the bsub commands to be executed
sub _generate_bsub_commands { ## no critic (Subroutines::ProhibitExcessComplexity)
  my ( $self, $outer_arg_refs ) = @_;

  my @bsub_commands;

  my $arg_refs = $outer_arg_refs->{arg_refs};

  my @positions = @{ $outer_arg_refs->{positions} };
  my $multiplex = $outer_arg_refs->{multiplex};

  my $required_job_completion = $arg_refs->{required_job_completion};

  my $id_run                  = $self->id_run();

  $self->clear_lane_tag_pairs();

  my $input_path  = $self->recalibrated_path(). q{/};
  my $output_path = $self->archive_path() . q{/};

  my $lsf_array_string;
  my $bam_filename = $id_run . q{_};

  if ( $multiplex ) {

    foreach my $position ( @positions ) {
      $self->add_lane_tag_pairs( $position );
    }

    $lsf_array_string = $self->lsf_job_array_from_lane_tag_pairs();
    $input_path .= q{lane} . $self->position_decode_string() . q{/};
    $output_path .= q{lane} . $self->position_decode_string() . q{/};
    $bam_filename .=  $self->position_decode_string();
    $bam_filename .= q{#} . $self->tag_index_decode_string();

  } else {

    $lsf_array_string = $self->create_array_string( @positions );
    $bam_filename .= $self->lsb_jobindex();
  }

  my $log_folder = $self->make_log_dir( $self->archive_path() );

  my $outfile = $log_folder . q{/} . $self->bsub_output_file();

  my $job_name = q{bam_alignment} . q{_} . $id_run . q{_} . $self->timestamp();

  my $out_subs_name = $job_name . q{.%I.%J};
  $outfile =~ s/substitute_this/$out_subs_name/xms;

  $job_name .= $lsf_array_string;
  if ( ! $self->no_array_cpu_limit() ) {
    $job_name .= q{%} . $self->array_cpu_limit();
  }
  $job_name = q{'} . $job_name . q{'};

  my $input_bam     = $input_path . $bam_filename . q{.bam};
  my $output_prefix = $output_path . $bam_filename;


  my $job_command = qq{$BAM_ALIGNMENT_SCRIPT} . qq{ --id_run $id_run --position };

  if ( $multiplex ) {
      $job_command .= $self->position_decode_string();
      $job_command .= q{ --tag_index } . $self->tag_index_decode_string();
  } else {
      $job_command .= $self->lsb_jobindex();
  }

  $job_command .= qq{ --input $input_bam};
  $job_command .= qq{ --output_prefix $output_prefix};
  $job_command .= q{ --do_markduplicates};

  if($self->not_strip_bam_tag()){
     $job_command .= q{ --not_strip_bam_tag};
  }

  if($self->illumina2bam_jar_location()){
     $job_command .= q{ --illumina2bam_jar_location } . $self->illumina2bam_jar_location();
  }

  if($self->is_paired_read()){
      $job_command .= q{ --is_paired_read};
  }else{
      $job_command .= q{ --nois_paired_read};
  }

  my $resources = ( $self->fs_resource_string( {
      counter_slots_per_job => $self->pb_cal_pipeline_conf()->{io_resource_slots},
      resource_string => $self->_default_resources(),
    } ) );

  my $job_sub =  q{bsub -q } . $self->lsf_queue()
                .  q{ } . $self->ref_adapter_pre_exec_string()
                . qq{ $resources $required_job_completion -J $job_name -o $outfile '$job_command'};

  if ( $self->verbose() ) { $self->log($job_sub); }


  return $job_sub;
}

sub _default_resources {
  my ( $self ) = @_;
  my $mem = $self->general_values_conf()->{bam_creation_memory};
  return qq{-R 'select[mem>$mem] rusage[mem=$mem] span[hosts=1]' -M${mem}000 -n} . $self->pb_cal_pipeline_conf()->{bwa_aln_threads};
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::bam_alignment

=head1 VERSION

$LastChangedRevision: 15858 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::bam_alignment->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating bam file.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the fastq files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=item npg_common::roles::run::lane::tag_info

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
