#############
# $Id: illumina2bam.pm 16492 2013-01-15 17:16:26Z ib5 $
# Created By: gq1
# Last Maintained By: $Author: ib5 $
# Created On: 2011-05-16
# Last Changed On: $Date: 2013-01-15 17:16:26 +0000 (Tue, 15 Jan 2013) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/archive/file/generation/illumina2bam.pm $

package npg_pipeline::archive::file::generation::illumina2bam;

use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;
use Perl6::Slurp;

extends q{npg_pipeline::archive::file::generation};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 16492 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $DEFAULT_RESOURCES => q{-R 'select[mem>2500] rusage[mem=2500]' -M2500000};
Readonly::Scalar our $JAVA_CMD          =>q{/software/bin/java};

sub generate {
  my ( $self, $arg_refs ) = @_;

  my @positions = $self->positions( $arg_refs );

  $self->log( q{Creating Jobs to run illumina2bam for run } . $self->id_run );

  my $alims = $self->lims->associated_child_lims_ia;
  my @job_ids;
  for my $p (@positions){
    my $bsub_cmd = $self->_generate_bsub_commands( $arg_refs, $alims->{$p});
    push @job_ids, $self->submit_bsub_command( $bsub_cmd );
  }

  return @job_ids;
}

has q{_illumina2bam_cmd} => (isa        => q{Str},
                              is         => q{ro},
                              lazy_build => 1,
                             );

sub _build__illumina2bam_cmd {

   my $self = shift;

   return $JAVA_CMD . q{ -Xmx1024m}
                    . q{ -jar } . $self->illumina2bam_jar_location() . q{/Illumina2bam.jar}
                    . q{ CREATE_MD5_FILE=true}
}

has q{_bam_adapter_detect_cmd} => (isa        => q{Str},
                                 is         => q{ro},
                                 lazy_build => 1,
                                );
sub _build__bam_adapter_detect_cmd {

   my $self = shift;

   return $JAVA_CMD . q{ -Xmx1024m}
                    . q{ -jar } . $self->illumina2bam_jar_location() . q{/BamAdapterFinder.jar}
                    . q{ ADAPTER_LENGTH_TAG=a3 ADAPTER_MATCH_TAG=ah}
                    . q{ CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT}
}

has q{_bam_index_decode_cmd} => (isa        => q{Str},
                                 is         => q{ro},
                                 lazy_build => 1,
                                );
sub _build__bam_index_decode_cmd {

   my $self = shift;

   return $JAVA_CMD . q{ -Xmx1024m}
                    . q{ -jar } . $self->illumina2bam_jar_location() . q{/BamIndexDecoder.jar}
                    . q{ CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT}
}

sub _generate_bsub_commands {
  my ( $self, $arg_refs, $lane_lims ) = @_;

  my $position = $lane_lims->position;
  my $required_job_completion = $arg_refs->{required_job_completion};

  my $id_run         = $self->id_run();
  my $intensity_path = $self->intensity_path();
  my $bam_basecall_path  = $self->bam_basecall_path();

  my $full_bam_name  = $bam_basecall_path . q{/}. $id_run . q{_} .$position. q{.bam};

  my $job_name = q{illumina2bam_} . $id_run . q{_} . $position. q{_} . $self->timestamp();

  my $log_folder = $self->make_log_dir( $bam_basecall_path );
  my $outfile = $log_folder . q{/} . $self->bsub_output_file();
  my $out_subs_name = $job_name . q{.%J};
  $outfile =~ s/substitute_this/$out_subs_name/xms;

  $job_name = q{'} . $job_name . q{'};

  my $job_command = $self->_illumina2bam_cmd()
                  . q{ I=} . $intensity_path
                  . q{ L=} . $position
		  . q{ B=} . $self->basecall_path();

  my $st_names = $self->_get_library_sample_study_names($lane_lims);

  if($st_names->{library}){
      $job_command .= q{ LIBRARY_NAME="} . $st_names->{library} . q{"};
  }
  if($st_names->{sample}){
      $job_command .= q{ SAMPLE_ALIAS="} . $st_names->{sample} . q{"};
  }
  if($st_names->{study}){
      my $study = $st_names->{study};
      $study =~ s/"/\\"/gmxs;
      $study =~ s/'/'"'"'/gmxs;
      $job_command .= q{ STUDY_NAME="} . $study . q{"};
  }
  if ($self->extra_tradis_transposon_read) {
      $job_command .= ' SEC_BC_SEQ=BC SEC_BC_QUAL=QT BC_SEQ=tr BC_QUAL=tq';
  }

  my $read1_index_special_circumstances = 0;
  if ($self->_special_read1_index_exists($lane_lims)) {
      $read1_index_special_circumstances = 1;
      $job_command .= ' FIRST_INDEX=5 FINAL_INDEX=10 FIRST=11 FINAL=75 FIRST=84 FINAL=158 FIRST_INDEX=1 FINAL_INDEX=4 SEC_BC_SEQ=BR SEC_BC_QUAL=QR ';
  }

  if ( $self->is_paired_read() && !$read1_index_special_circumstances){
      # omit BamAdapterFinder for special read1 index
      $job_command .= q{ OUTPUT=} . q{/dev/stdout}
                   . q{ COMPRESSION_LEVEL=0};
      $job_command .= q{ | }
      . $self->_bam_adapter_detect_cmd()
          . q{ I=/dev/stdin };
  }

  if( $self->is_multiplexed_lane($position) ){
      my $tag_list_file = $self->runfolder_path() . qq{/lane_$position.taglist};
      $job_command .= q{ OUTPUT=} . q{/dev/stdout}
                   . q{ COMPRESSION_LEVEL=0};
      $job_command .= q{ | }
                   . $self->_bam_index_decode_cmd()
                   . q{ I=/dev/stdin }
                   . q{ BARCODE_FILE=} . $tag_list_file
                   . q{ METRICS_FILE=} . $full_bam_name . q{.tag_decode.metrics};
      my $num_of_plexes_per_lane = $self->_get_number_of_plexes_excluding_control($tag_list_file, $lane_lims);
      if( $num_of_plexes_per_lane == 1 ){
         $job_command .= q{ MAX_NO_CALLS=} . $self->pb_cal_pipeline_conf()->{single_plex_decode_max_no_calls};
         $job_command .= q{ CONVERT_LOW_QUALITY_TO_NO_CALL=true};
      }
  }

  $job_command = q{set -o pipefail;} . $job_command;
  $job_command .= q{ OUTPUT=} .  $full_bam_name;

  my $resources = ( $self->fs_resource_string( {
      counter_slots_per_job => $self->pb_cal_pipeline_conf()->{io_resource_slots},
      resource_string => $DEFAULT_RESOURCES,
  } ) );

  my $job_sub = q{bsub -q } . $self->lsf_queue() . qq{ $resources $required_job_completion -J $job_name -o $outfile '$job_command'};

  if ( $self->verbose() ) { $self->log($job_sub); }

  return $job_sub;
}

sub _get_library_sample_study_names {

   my ($self, $lane_lims) = @_;

   my $names = $self->get_study_library_sample_names($lane_lims);

   my ($study_names, $library_names, $sample_names);

   if($names->{study}){
      $study_names = join q{,}, @{$names->{study}};
   }
   if($names->{library}){
      $library_names = join q{,}, @{$names->{library}};
   }
   if($names->{sample}){
      $sample_names = join q{,}, @{$names->{sample}};
   }

   my $joined_names = {study=>$study_names, library=>$library_names, sample=>$sample_names};

   return $joined_names;
}

sub _get_number_of_plexes_excluding_control {
    my ($self, $tag_list_file, $lane_lims) = @_;

    my @lines = slurp $tag_list_file, {chomp=>1};
    shift @lines;

    my $spiked_phix_tag_index = $lane_lims->spiked_phix_tag_index();
    $spiked_phix_tag_index ||= 0;

    my $count = 0;
    foreach my $line (@lines){

       my ($tag_index) = $line =~ /^[ATGC]+\t(\d+)\t/mxs;
       if(defined $tag_index && $tag_index != $spiked_phix_tag_index){
          $count++;
       }
    }

    return $count;
}

sub _special_read1_index_exists {
    # Check for library type with "special" index near start of read 1
    # If present, assume cycles 1-4 are a "second index" to be stored for later reference, 5-10 are the "special" index to be used for demultiplexing, 11-75 and 84-158 are reads 1 and 2 of normal sequence respectively, and 76-83 are the "standard" indexing read to be omitted.
    # Possible extension:  Replace hard-coded cycle ranges with lookup in a database or config file.
    my ($self, $lane_lims) = @_;
    my @special_types = ('jecfoo',
                         '3 prime poly-A pulldown',
        );
    my %special;
    foreach my $lib (@special_types) { $special{$lib} = 1; }
    my $has_index = 0;
    foreach my $lib ($lane_lims->library_types()) {
        if ($special{$lib}) {
            $has_index = 1;
            last;
        }
    }
    return $has_index;
}

no Moose;

__PACKAGE__->meta->make_immutable;

1;
__END__

=head1 NAME

npg_pipeline::archive::file::generation::illumina2bam

=head1 VERSION

$LastChangedRevision: 16492 $

=head1 SYNOPSIS

  my $oAfgfq = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which knows how to construct and submits the command line to LSF for creating bam files from bcl files.

=head1 SUBROUTINES/METHODS

=head2 generate - generates the bsub jobs and submits them for creating the fastq files, returning an array of job_ids.

  my @job_ids = $oAfgfq->generate({
    required_job_completion} => q{-w (123 && 321)};
    timestamp                => q{20090709-123456};
    solexa_bin               => q{/software/solexa/bin};
    id_run                   => 1234;
    BSUB_ERROR_FILE          => q{substitute_this.err};
    BSUB_OUTPUT_FILE         => q{substitute_this.out};
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ib5 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Guoying Qi (gq1@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
