#############
# $Id: touchCompletedLane.pm 15127 2012-02-07 16:01:17Z mg8 $
# Created By: ajb
# Last Maintained By: $Author: mg8 $
# Created On: 2010-09-02
# Last Changed On: $Date: 2012-02-07 16:01:17 +0000 (Tue, 07 Feb 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/archive/file/generation/touchCompletedLane.pm $

package npg_pipeline::archive::file::generation::touchCompletedLane;
use Moose;
use Carp;
use English qw{-no_match_vars};
use Readonly;

extends q{npg_pipeline::archive::file::generation};

Readonly::Scalar our $VERSION  => do { my ($r) = q$LastChangedRevision: 15127 $ =~ /(\d+)/mxs; $r; };

Readonly::Scalar our $SCRIPT_NAME => q{npg_pipeline_touch_completed_lane};

=head1 NAME

npg_pipeline::archive::file::generation::touchCompletedLane

=head1 VERSION

$LastChangedRevision: 15127 $

=head1 SYNOPSIS
  my $oAfgtcl = npg_pipeline::archive::file::generation::touchCompletedLane->new({
    run_folder => $sRunFolder,
  });

=head1 DESCRIPTION

Object module which will go and touch a file in the archive directory to show that the lane has had a complete set of post_qseq jobs run on it

=head1 SUBROUTINES/METHODS

=head2 touch_file

Touches the file for each lane specified

=cut

sub touch_file {
  my ( $self, $arg_refs ) = @_;

  my $archive_path = $self->archive_path();

  my @positions = $self->positions( $arg_refs );

  eval {
    foreach my $lane ( @positions ) {
      my $output = qx{touch $archive_path/completed_lane_$lane};
      if ( $CHILD_ERROR ) {
        croak qq{unable to touch completed_lane_$lane: $output};
      }
    }
    1;
  } or do {
    croak $EVAL_ERROR;
  };

  return 1;
}

=head2 launch_bsub_job

generate a bsub job and submit it to touch the completed file

=cut

sub launch_bsub_job {
  my ( $self, $arg_refs ) = @_;

  my @job_ids;
  my @positions = $self->positions( $arg_refs );

  foreach my $lane ( @positions ) {
    $arg_refs->{lane} = $lane;
    my $bsub_command = $self->_generate_bsub_command( $arg_refs );
    push @job_ids, $self->submit_bsub_command( $bsub_command );
  }

  return @job_ids;
}

sub _generate_bsub_command {
  my ( $self, $arg_refs ) = @_;

  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp   = $self->timestamp();
  my $script_bin  = $self->internal_script_bin();
  my $id_run      = $self->id_run();

  my $outfile     = $self->make_log_dir( $self->archive_path() ) . q{/} . $self->bsub_output_file();
  my $job_name    = q{touch_secondary_completed_file_} . $id_run . q{_} . $arg_refs->{lane} . q{_} . $timestamp;
  my $replacement = $job_name . q{.%J};
  $outfile        =~ s/substitute_this/$replacement/xms;
  my $options = join q[ ], q[--id_run], $self->id_run, q[--archive_path], $self->archive_path, q[--lanes] , $arg_refs->{lane};
  my $job_sub = q{bsub -q } . $self->small_lsf_queue() . q{ };
  $job_sub .=  ( $self->fs_resource_string( {
    counter_slots_per_job => 1,
  } ) ) . q{ };
  $job_sub .= qq{$required_job_completion -J $job_name -o $outfile '$script_bin/} . $SCRIPT_NAME . q{ } . $options . q{'};
  if ( $self->verbose() ) { $self->log( $job_sub ); }
  return $job_sub;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Carp

=item English -no_match_vars

=item Readonly

=item Moose

=item npg_pipeline::archive::file::generation

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2011 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
