#############
# $Id: update.pm 16101 2012-10-17 09:37:15Z mg8 $
# Created By: ajb
# Last Maintained By: $Author: mg8 $
# Created On: 2009-07-07
# Last Changed On: $Date: 2012-10-17 10:37:15 +0100 (Wed, 17 Oct 2012) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/npg-pipeline/tags/release-39.0/lib/npg_pipeline/run/status/update.pm $

package npg_pipeline::run::status::update;
use Moose;
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 16101 $ =~ /(\d+)/mxs; $r; };

extends q{npg_pipeline::run::status};

with qw{
  npg_common::roles::run::status
};

Readonly::Scalar our $UPDATE_STATUS_SCRIPT  => q{npg_pipeline_update_run_status};

has q{status_one} => (isa => q{Str}, is => q{ro}, required => 1,
                      documentation => q{the status you wish the first id_run to have},);

sub update {
  my ($self) = @_;

  my $id_run = $self->id_run();
  my $status = $self->status_one();

  $self->update_run_status( {
    id_run => $id_run,
    status_desc => $status,
  } );

  return 1;
}

###############
# responsible for generating the bsub command to be executed
sub _generate_bsub_command {
  my ($self, $arg_refs) = @_;
  my $required_job_completion = $arg_refs->{required_job_completion};
  my $timestamp = $self->timestamp();

  my $run_folder = $self->run_folder();
  my $status     = $self->status_one();
  my $id_run     = $self->id_run();

  my $status_with_underscores = $self->status_one();
  $status_with_underscores =~ s/[ ]/_/gxms;

  my $run_folder_loc = $self->runfolder_path();
  if ($status eq q{qc complete} || $status eq q{run archived}) {
    $run_folder_loc =~ s/analysis/outgoing/xms;
  }
  my $options = qq[--id_run $id_run --status_one "$status"];
  my $job_name = q{update_run_status_} . $id_run . q{_} . $status_with_underscores . q{_} . $timestamp;

  my $bsub_command = qq{bsub $required_job_completion -J $job_name -q } . $self->small_lsf_queue() . q{ };

  if ( $status eq q{analysis complete} ) {
    $bsub_command .= $self->files_present_pre_exec_string() . q{ };
  }
  $bsub_command .=  q{-o } . $run_folder_loc . q{/} . qq{$job_name.out };
  $bsub_command .=  q{'} . $self->internal_script_bin() . q{/} . $UPDATE_STATUS_SCRIPT . q{ };
  $bsub_command .= qq{$options'};
  if ($self->verbose()) {
    $self->log($bsub_command);
  }
  return $bsub_command;
}

sub submit_update {
  my ($self, $arg_refs) = @_;
  my $cmd = $self->_generate_bsub_command($arg_refs);
  my $job_id = $self->submit_bsub_command($cmd);
  return $job_id;
}

no Moose;
__PACKAGE__->meta->make_immutable;
1;
__END__

=head1 NAME

npg_pipeline::run::status::update

=head1 VERSION

$LastChangedRevision: 16101 $

=head1 SYNOPSIS

  my $rsu = npg_pipeline::run::status::update->new({
    run => npg::api::run object,
    id_run => $iIdRun,
    status_one => $sRunStatusDescription,
    status_two => $sRunStatusDescription, (optional, if run is paired with two runfolders)
  });

  $rsu->update();

=head1 DESCRIPTION

Class to update the run status, of a pair of runs if required. It can do this directly,
or submit a job to LSF so that it can be done at an appropriate time.

=head1 SUBROUTINES/METHODS

=head2 update - method to call to update a run's status

=head2 submit_update - handles generating and submitting an LSF job to do the update at the appropriate time

  my $job_id = $rsu->submit_update({
    required_job_completion => $sRequiredJobCompletion,
    timestamp => $sTimeStamp,
  });

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item npg_pipeline::run::status

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: mg8 $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
