# $Id: 10-base.t 16101 2012-10-17 09:37:15Z mg8 $
use strict;
use warnings;
use Test::More tests => 30;
use Test::Exception;
use t::util;

use_ok('npg_pipeline::base');

{
  throws_ok {npg_pipeline::base->new(no_bsub => 3)} qr/Validation failed for 'Bool' failed with value 3/, 'error trying to set boolean flag to 3';
  my $base = npg_pipeline::base->new();
  isa_ok($base, q{npg_pipeline::base});
  is($base->no_bsub, undef, 'no_bsub flag value is undefined if not set');
  ok(!$base->no_bsub, '   ... and it evaluates to false');
  is($base->local, 0, 'local flag is 0');
  $base = npg_pipeline::base->new(no_bsub => q[]);
  is($base->no_bsub, q[], 'no_bsub flag value is empty string as set');
  ok(!$base->no_bsub, '   ... and it evaluates to false');
  $base = npg_pipeline::base->new(no_bsub => 0);
  is($base->no_bsub, 0, 'no_bsub flag value is 0 as set');
  ok(!$base->no_bsub, '   ... and it evaluates to false');
  is($base->local, 0, 'local flag is 0');
  $base = npg_pipeline::base->new(no_bsub => 1);
  is($base->no_bsub, 1, 'no_bsub flag value is 1 as set');
  ok($base->no_bsub, '   ... and it evaluates to true');
  is($base->local, 1, 'local flag is 1');
  $base = npg_pipeline::base->new(local => 1);
  is($base->local, 1, 'local flag is 1 as set');
  $base = npg_pipeline::base->new(no_bsub => 0, local => 1);
  is($base->local, 1, 'local flag is 1 as set');
  is($base->no_bsub, 0, 'no_sub flag is 0 as set');
}

{
  my $base = npg_pipeline::base->new();
  ok(!$base->olb, 'OLB preprocessing is switched off by default');
  $base = npg_pipeline::base->new(olb => 1);
  ok($base->olb, 'OLB preprocessing is switched on as set');
}

{
  my $base;
  lives_ok {
    $base = npg_pipeline::base->new({
      conf_path => q{data/config_files},
      domain => q{test},
    });
  } q{base ok};

  foreach my $config_group ( qw{
    external_script_names_conf
    filesystem_locations_conf
    function_order_conf
    general_values_conf
    illumina_pipeline_conf
    pb_cal_pipeline_conf
  } ) {
    isa_ok( $base->$config_group(), q{HASH}, q{$} . qq{base->$config_group} );
  }
}

{
  my $path = t::util->new()->temp_directory;
  my $base = npg_pipeline::base->new( id_run => 7440, runfolder_path => $path);
  my $arg = q{-R 'select[mem>2500] rusage[mem=2500]' -M2500000};
  is ($base->fs_resource_string({resource_string => $arg,}), q[-R 'select[mem>2500] rusage[mem=2500,nfs_12=8]' -M2500000], 'resource string with sf resource');
  is ($base->fs_resource_string({resource_string => $arg, seq_irods => 1,}), q[-R 'select[mem>2500] rusage[mem=2500,nfs_12=8,seq_irods=1]' -M2500000], 'resource string with sf and irods resource');
  $base = npg_pipeline::base->new(id_run => 7440, runfolder_path => $path , no_sf_resource => 1);
  is ($base->fs_resource_string({resource_string => $arg,}), $arg, 'resource string with no sr resource if no_sf_resource is set');

  $arg = q{-R 'select[mem>13800] rusage[mem=13800] span[hosts=1]'};
  is ($base->fs_resource_string({resource_string => $arg, counter_slots_per_job => 8,}), $arg, 'resource string with no sr resource if no_sf_resource is set');
}

1;
