
# $Id: 10-pluggable_harold_pb_cal_bam.t 16492 2013-01-15 17:16:26Z ib5 $
use strict;
use warnings;
use Test::More tests => 19;
use Test::Exception;
use Test::Deep;
use t::util;
use Cwd qw/getcwd/;

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();

BEGIN {
  use_ok(q{npg_pipeline::pluggable::harold::PB_cal_bam});
}

my $runfolder_path = $util->analysis_runfolder_path();

{
  $util->set_staging_analysis_area();
  my $pb_cal_pipeline;
  lives_ok {
    $pb_cal_pipeline = npg_pipeline::pluggable::harold::PB_cal_bam->new({
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak creating new object};
  isa_ok($pb_cal_pipeline, q{npg_pipeline::pluggable::harold::PB_cal_bam}, q{$pb_cal_pipeline});

  my $expected_function_order = [ qw{
          lsf_start
          spider
          make_multiplex_lane_specific_tag_files
          create_archive_directory
          create_empty_fastq
          create_summary_link_analysis
          bustard_matrix_lanes
          bustard_matrix_all
          bustard_phasing_lanes
          bustard_phasing_all
          bustard_basecalls_lanes
          bustard_basecalls_all
          illumina_basecall_stats
          illumina2bam
          qc_tag_metrics
          harold_alignment_files
          harold_calibration_tables
          harold_recalibration
          split_bam_by_tag
          bam2fastqcheck_and_cached_fastq
          qc_qX_yield
          qc_adapter
          qc_insert_size
          qc_sequence_error
          qc_gc_fraction
          qc_ref_match
          bam_alignment
          bam_cluster_counter_check
          qc_gc_bias
          qc_pulldown_metrics
          qc_genotype
          touch_completed_lane
          lsf_end     
  }];
  is_deeply( $pb_cal_pipeline->function_order() , $expected_function_order, q{Function order correct} );
}

{
  my $pb_cal_pipeline;
  lives_ok {
    $pb_cal_pipeline = npg_pipeline::pluggable::harold::PB_cal_bam->new({
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      no_recalibration => 1,
      no_bsub => 1,
    });
  } q{no croak creating new object};

  ok( !scalar $pb_cal_pipeline->harold_alignment_files(),     q{no_recalibration - no alignment files launched} );
  ok( !scalar $pb_cal_pipeline->harold_calibration_tables(),  q{no_recalibration - no calibration tables launched} );
 
  lives_ok { $pb_cal_pipeline->prepare() } 'prepare lives';
  ok( $pb_cal_pipeline->illumina_basecall_stats(),  q{olb false - illumina_basecall_stats job launched} );
  ok( !$pb_cal_pipeline->bustard_matrix_lanes(),  q{olb false - bustard_matrix_lanes job is not launhed} );
}

{
  my $pb_cal_pipeline = npg_pipeline::pluggable::harold::PB_cal_bam->new(
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      no_bsub => 1,
      olb => 1
  );
  ok( !$pb_cal_pipeline->illumina_basecall_stats(),  q{olb true - illumina_basecall_stats job is not launched} );
}

my $analysis_path = q{Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto};

{
  my $pb;
  lives_ok {
    $pb = npg_pipeline::pluggable::harold::PB_cal_bam->new({
      id_run => 1234,
      function_order => [qw(qc_qX_yield illumina2bam qc_insert_size)],
      run_folder => q{123456_IL2_1234},
      analysis_path => $analysis_path,
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak on creation};
  $util->set_staging_analysis_area({with_latest_summary => 1});
  is(join(q[ ], @{$pb->function_order()}), 'lsf_start qc_qX_yield illumina2bam qc_insert_size lsf_end', 'function_order set on creation');
}

{
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  my $pb;
  $util->set_staging_analysis_area();
  lives_ok {
    $pb = npg_pipeline::pluggable::harold::PB_cal_bam->new({
      script_name => q{test},
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      function_order => [qw{illumina2bam qc_qX_yield qc_adapter update_warehouse qc_insert_size archive_to_irods}],
      lane => 4,
      analysis_path => $analysis_path,
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      no_bsub => 1,
      repository => 't/data/sequence',
    });
  } q{no croak on new creation};
  mkdir $pb->archive_path;
  mkdir $pb->qc_path;

  lives_ok { $pb->main() } q{no croak running qc->main()};
  my $timestamp = $pb->timestamp;
  my $recalibrated_path = $pb->recalibrated_path();
  my $log_dir = $pb->make_log_dir( $recalibrated_path );
  is($pb->_update_warehouse_command, qq[bsub -q test  -J whupdate_1234_pb_cal_bam -o $log_dir/whupdate_1234_pb_cal_bam_] . $timestamp . q[.out 'warehouse_loader --id_run 1234'], 'update warehouse command');

  is ($pb->_and_preexecs(q[-E 'npg_pipeline_preexec_references'], '[ -d t ]'), q[-E 'npg_pipeline_preexec_references && [ -d t ]'], 'preexecs combined correctly');

  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  my $cwd = getcwd();
  my $expected = qq(bsub -q test -m "sf6_1 sf5_2" -R 'rusage[nfs_12=2]'  -J tileviz_1234_${timestamp}[4] -o ${runfolder_path}/Data/Intensities/BAM_basecalls_${timestamp}/PB_cal_bam/archive/qc/log/tileviz_1234_${timestamp}.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence && [ -d ${runfolder_path}/Data/Intensities/BAM_basecalls_${timestamp}/PB_cal_bam/archive/qc ]' 'numactl --interleave=all -- t/bin/software/solexa/bin/pb_calibration/v9.0/tileviz --bam_in ${runfolder_path}/Data/Intensities/BAM_basecalls_${timestamp}/PB_cal_bam/1234_) . q[`echo $LSB_JOBINDEX`] . qq[.bam --dir_out ${runfolder_path}/Data/Intensities/BAM_basecalls_${timestamp}/PB_cal_bam/archive/qc/tileviz --snp_file ${cwd}/t/data/sequence/references/PhiX/default/all/snps/phix-illumina.fa.rod --mismatch_tool_path t/bin/software/solexa/bin/pb_calibration/v9.0/spatial_filter --num_tiles 120'];
  my @out = $pb->_tileviz_bsub;
  is(shift @out, $expected, 'tileviz bsub');
  is(shift @out,  "${runfolder_path}/Data/Intensities/BAM_basecalls_${timestamp}/PB_cal_bam/archive/qc/tileviz", 'tileviz output dir');
}

1;
