# $Id: 10-pluggable_harold_post_qc_review.t 16101 2012-10-17 09:37:15Z mg8 $
use strict;
use warnings;
use English qw{-no_match_vars};
use Test::More tests => 22;
use Test::Deep;
use Test::Exception::LessClever;
use t::util;
use npg::api::request;

my $util = t::util->new();
$util->catch_email();
my $conf_path = $util->conf_path();

my $runfolder_path = $util->analysis_runfolder_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$PROGRAM_NAME = q{npg_pipeline_post_qc_review};

use_ok('npg_pipeline::pluggable::harold::post_qc_review');

{
  my $post_qc_review;
  my @functions_in_order = qw(
    set_run_status_archival_in_progress
    archive_to_irods
    upload_illumina_analysis_to_qc_database
    upload_auto_qc_to_qc_database
    move_to_outgoing
    set_run_status_run_archived
    set_run_status_qc_complete
    update_warehouse
    );
  my @original = @functions_in_order;
  unshift @original, 'lsf_start';
  push @original, 'lsf_end';
  $util->set_staging_analysis_area({with_latest_summary => 1});

  lives_ok {
    $post_qc_review = npg_pipeline::pluggable::harold::post_qc_review->new(
      id_run => 1234,
      function_order => \@functions_in_order,
      runfolder_path => $runfolder_path,
      run_folder => q{123456_IL2_1234},
      verbose => 1,
      analysis_path => q{Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto},
      conf_path => $conf_path,
      domain => q{test},
      no_spider => 1,
    );
  } q{no croak on creation};
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  lives_ok { $post_qc_review->main(); } q{no croak running harold->main()};
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[];
  isa_ok($post_qc_review, q{npg_pipeline::pluggable::harold::post_qc_review}, q{$post_qc_review});
  is(join(q[ ], @{$post_qc_review->function_order()}), join(q[ ], @original), q{$post_qc_review->function_order() set on creation});

  my $timestamp = $post_qc_review->timestamp;
  my $recalibrated_path = $post_qc_review->recalibrated_path();
  my $log_dir = $post_qc_review->make_log_dir( $recalibrated_path );

  is($post_qc_review->_update_warehouse_command, qq[bsub -q test  -J whupdate_1234_post_qc_review -o $log_dir/whupdate_1234_post_qc_review_] . $timestamp . q[.out 'warehouse_loader --id_run 1234'], 'update warehouse command');

  $log_dir = $post_qc_review->make_log_dir( $runfolder_path );
  is($post_qc_review->_interop_command, qq[bsub -q test  -J interop_1234_post_qc_review -R 'rusage[nfs_12=1,seq_irods=15]' -o $log_dir/interop_1234_post_qc_review_] . $timestamp . qq[.out 'irods_interop_loader.pl --id_run 1234 --runfolder_path $runfolder_path'], 'irods_interop_loader.pl command');
  

  my $webcache = $recalibrated_path . q{/archive/webcache};
  qx{mkdir -p $webcache};

  lives_ok {$post_qc_review->spider();} q{spider runs ok};
  is( $ENV{ npg::api::request->cache_dir_var_name() }, $webcache, q{environment variable set to webcache} );
  $ENV{ npg::api::request->cache_dir_var_name() } = q{};

  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
  lives_ok {$post_qc_review->spider();} q{spider runs ok};
  is( $ENV{ npg::api::request->cache_dir_var_name() },q[t/data] , q{environment variable as set} );
}

{
  my $p = npg_pipeline::pluggable::harold::post_qc_review->new(
      id_run => 1234,
      runfolder_path => $runfolder_path,
      run_folder => q{123456_IL2_1234},
      no_irods_archival => 1,
      no_warehouse_update => 1,
    );
  ok(!$p->archive_to_irods(), 'archival to irods switched off');
  ok(!$p->update_warehouse(), 'update to warehouse switched off');
  is($p->no_status_updates,0, 'status_update switched on');
}

{
  my $p = npg_pipeline::pluggable::harold::post_qc_review->new(
      id_run => 1234,
      runfolder_path => $runfolder_path,
      run_folder => q{123456_IL2_1234},
      local => 1,
    );
  ok(!$p->archive_to_irods(), 'archival to irods switched off');
  ok(!$p->update_warehouse(), 'update to warehouse switched off');
  is($p->no_summary_link,1, 'summary_link switched off');
  is($p->no_status_updates,1, 'status_update switched off');
}

{
  my $p = npg_pipeline::pluggable::harold::post_qc_review->new(
      id_run => 1234,
      runfolder_path => $runfolder_path,
      run_folder => q{123456_IL2_1234},
      local => 1,
      no_warehouse_update => 0,
	  no_status_updates => 0,
    );
  ok(!$p->archive_to_irods(), 'archival to irods switched off');
  ok($p->update_warehouse(), 'update to warehouse switched on');
  is($p->no_summary_link,1, 'summary_link switched off');
  is($p->no_status_updates,0, 'status_update switched on');
}

1;
