# $Id: 20-archive_file_generation-bam_alignment.t 15759 2012-06-22 13:05:10Z gq1 $
use strict;
use warnings;
use Test::More tests => 6;
use Test::Exception;
use t::util;

use_ok('npg_pipeline::archive::file::generation::bam_alignment');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

my $analysis_runfolder_path = $util->analysis_runfolder_path();

{
  $util->create_analysis( { no_touch_files => 1, bam_generation => 1, } );
  `cp t/data/runfolder/Data/RunInfo.xml $analysis_runfolder_path/`;


  my $bam_alignment;

  lives_ok { $bam_alignment = npg_pipeline::archive::file::generation::bam_alignment->new({
    run_folder => q{123456_IL2_8033},
    runfolder_path => $analysis_runfolder_path,
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    repository => q{t/data/sequence},
  }); } q{no croak creating $fastq_generator object};

  isa_ok($bam_alignment, q{npg_pipeline::archive::file::generation::bam_alignment}, q{$bam_alignment});

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    lane    => [1,2,5,7,8],
  };

  my $bsub_command_lanes = $bam_alignment->_generate_bsub_commands({
      positions => [1,2],
      multiplex => 0,
      arg_refs => $arg_refs,
  });

  $bsub_command_lanes = $util->drop_temp_part_from_paths( $bsub_command_lanes );
  my $expected_command_lanes = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n6 -w'done(123) && done(321)' -J 'bam_alignment_8033_20090709-123456[1-2]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/bam_alignment_8033_20090709-123456.%I.%J.out '/software/solexa/bin/bam_alignment.pl --id_run 8033 --position `echo $LSB_JOBINDEX` --input /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/8033_`echo $LSB_JOBINDEX`.bam --output_prefix /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/8033_`echo $LSB_JOBINDEX` --do_markduplicates --illumina2bam_jar_location t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00 --nois_paired_read'};
  is( $bsub_command_lanes, $expected_command_lanes, q{lane bsub command is correct});
  

  my $runfolder = $bam_alignment->runfolder_path();
  `cp t/data/tag_files/lane_7.tag $runfolder`;
  `cp t/data/tag_files/lane_7.tag $runfolder/lane_8.tag`;
  
  my $bsub_command_plex = $bam_alignment->_generate_bsub_commands({
      positions => [7,8],
      multiplex => 1,
      arg_refs => $arg_refs,
  });

  $bsub_command_plex = $util->drop_temp_part_from_paths( $bsub_command_plex );
  my $expected_command_plex = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n6 -w'done(123) && done(321)' -J 'bam_alignment_8033_20090709-123456[7000,7002,7012,8000,8002,8012]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/log/bam_alignment_8033_20090709-123456.%I.%J.out '/software/solexa/bin/bam_alignment.pl --id_run 8033 --position `echo $LSB_JOBINDEX/1000 | bc` --tag_index `echo $LSB_JOBINDEX%1000 | bc` --input /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/lane`echo $LSB_JOBINDEX/1000 | bc`/8033_`echo $LSB_JOBINDEX/1000 | bc`#`echo $LSB_JOBINDEX%1000 | bc`.bam --output_prefix /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`/8033_`echo $LSB_JOBINDEX/1000 | bc`#`echo $LSB_JOBINDEX%1000 | bc` --do_markduplicates --illumina2bam_jar_location t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00 --nois_paired_read'};
  is( $bsub_command_plex, $expected_command_plex, q{plex bsub command is correct});

  my @job_ids = $bam_alignment->generate($arg_refs);
  is(scalar @job_ids, 2, "2 jobs submitted, one for inedexed lanes and one for non-indexed lanes");
}
1;
