# $Id: 20-archive_file_generation-cram.t 15874 2012-07-12 09:19:16Z kl2 $
use strict;
use warnings;
use Test::More tests => 6;
use Test::Exception;
use t::util;

use_ok('npg_pipeline::archive::file::generation::cram');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

my $analysis_runfolder_path = $util->analysis_runfolder_path();
my $analysis_archive_path = $analysis_runfolder_path . q{/archive};

{
  $util->create_analysis( { no_touch_files => 1, bam_generation => 1, } );

  my $cram_generator;

  lives_ok { $cram_generator = npg_pipeline::archive::file::generation::cram->new(
    {
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $analysis_runfolder_path,
      archive_path => $analysis_archive_path,
      timestamp => q{20090709-123456},
      repository => q{t/data/sequence},
  }); } q{no croak creating $cram_generator object};
  isa_ok($cram_generator, q{npg_pipeline::archive::file::generation::cram}, q{$cram_generator});

  my $arg_refs = {
    id_run => 1234,
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  my $hex;
  my $md5_object = Digest::MD5->new();
  my @bsub_commands = $cram_generator->_generate_bsub_commands(
    {
        1 => [ 15 ],
        2 => [],
        3 => [],
        4 => [ 16, 32 ],
    },
    $arg_refs->{required_job_completion},
  );
  is( scalar @bsub_commands, 2, q{2 bsub commands returned} );
  my $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[0] );
  my $expected_command = q{bsub -q srpipeline -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'bam2cram.pl_1234_20090709-123456[3,2]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/archive/log/bam2cram.pl_1234_20090709-123456.%I.%J.out '/software/solexa/bin/bam2cram.pl --id_run 1234  --position $LSB_JOBINDEX --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/archive'};
  is( $bsub_command, $expected_command, q{first generated bsub command is correct});

  $bsub_command = $util->drop_temp_part_from_paths( $bsub_commands[1] );
  $expected_command = q{bsub -q srpipeline -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M8000000 -n4 -w'done(123) && done(321)' -J 'bam2cram.pl_1234_20090709-123456[1015,4016,4032]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/archive/log/bam2cram.pl_1234_20090709-123456.%I.%J.out '/software/solexa/bin/bam2cram.pl --id_run 1234  --position `echo $LSB_JOBINDEX/1000 | bc`  --tag_index `echo $LSB_JOBINDEX%1000 | bc` --archive_path /nfs/sf45/IL2/analysis/123456_IL2_1234/archive'};
  is( $bsub_command, $expected_command, q{second generated bsub command is correct});
}

1;
