# $Id: 20-archive_file_generation-illumina2bam.t 16492 2013-01-15 17:16:26Z ib5 $

use strict;
use warnings;
use Test::More tests => 27;
use Test::Exception;
use Test::Differences;
use t::util;

use_ok('npg_pipeline::archive::file::generation::illumina2bam');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = 't/data';

{
  $util->create_analysis({no_touch_files => 1});
  $util->copy_tag_file(8);

  my $tag_list_file = $util->analysis_runfolder_path() . q{/lane_8.taglist};
  `cp t/data/tag_files/lane_8.taglist $tag_list_file`;
  my $runfolder = $util->analysis_runfolder_path() . '/';
  `cp t/data/runfolder/Data/RunInfo.xml $runfolder`;

  my $bam_generator;

  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => q{123456_IL2_1234},
    runfolder_path => $util->analysis_runfolder_path(),
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    id_run => 1234,
	extra_tradis_transposon_read => 1,
    bam_basecall_path => $util->analysis_runfolder_path() . q{/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls},
  }); } q{no croak creating bam_generator object};

  isa_ok($bam_generator, q{npg_pipeline::archive::file::generation::illumina2bam}, q{$bam_generator});
  is($bam_generator->extra_tradis_transposon_read, 1, 'TraDIS set');
  $bam_generator->extra_tradis_transposon_read(0);
  is($bam_generator->extra_tradis_transposon_read, 0, 'TraDIS not set');
  isa_ok($bam_generator->lims, 'st::api::lims', 'cached lims object');
  is($bam_generator->lims->batch_id, 2015, 'batch id from lims object');

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'}, 
  };
  
  my $alims = $bam_generator->lims->associated_child_lims_ia;
  my $position = 8;
  my $bsub_command = $bam_generator->_generate_bsub_commands( $arg_refs , $alims->{$position});

  is( $bam_generator->_get_number_of_plexes_excluding_control($tag_list_file, $alims->{$position}), 
  1, 'correct number of plexes');

  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  my $expected_cmd = q{bsub -q test -R 'select[mem>2500] rusage[mem=2500,nfs_12=4]' -M2500000 -w'done(123) && done(321)' -J 'illumina2bam_1234_8_20090709-123456' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/log/illumina2bam_1234_8_20090709-123456.%J.out 'set -o pipefail;/software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/Illumina2bam.jar CREATE_MD5_FILE=true I=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto L=8 B=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls LIBRARY_NAME="51021" SAMPLE_ALIAS="NA18907" STUDY_NAME="1000Genomes Project Pilot 1: 1000Genomes Project Pilot 1" OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamIndexDecoder.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  BARCODE_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/lane_8.taglist METRICS_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/1234_8.bam.tag_decode.metrics MAX_NO_CALLS=6 CONVERT_LOW_QUALITY_TO_NO_CALL=true OUTPUT=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/1234_8.bam'};
  eq_or_diff([split"=",$bsub_command], [split"=",$expected_cmd], 'correct bsub command for lane 8');
  
  my @jids;
  lives_ok { @jids = $bam_generator->generate($arg_refs); } q{no croak running generate};
  is(scalar @jids, 8, 'correct number of jobs submitted');  
  
  my $names = $bam_generator->_get_library_sample_study_names($alims->{1});
  is( $names->{study}, '1000Genomes Project Pilot 1: 1000Genomes Project Pilot 1', 'correct study name for lane 1');
  is( $names->{library}, '51021', 'correct library id for lane 1');
  is( $names->{sample}, 'NA18907', 'correct sample name for lane 1'); 


  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => q{123456_IL2_1234},
    runfolder_path => $util->analysis_runfolder_path(),
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    id_run => 1234,
    is_paired_read => 1,
    extra_tradis_transposon_read => 0,
    bam_basecall_path => $util->analysis_runfolder_path() . q{/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls},
  }); } q{no croak creating bam_generator object (paired read)};
  $bsub_command = $bam_generator->_generate_bsub_commands( $arg_refs , $alims->{$position});
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  $expected_cmd = q{bsub -q test -R 'select[mem>2500] rusage[mem=2500,nfs_12=4]' -M2500000 -w'done(123) && done(321)' -J 'illumina2bam_1234_8_20090709-123456' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/log/illumina2bam_1234_8_20090709-123456.%J.out 'set -o pipefail;/software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/Illumina2bam.jar CREATE_MD5_FILE=true I=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto L=8 B=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls LIBRARY_NAME="51021" SAMPLE_ALIAS="NA18907" STUDY_NAME="1000Genomes Project Pilot 1: 1000Genomes Project Pilot 1" OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamAdapterFinder.jar ADAPTER_LENGTH_TAG=a3 ADAPTER_MATCH_TAG=ah CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamIndexDecoder.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  BARCODE_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/lane_8.taglist METRICS_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/1234_8.bam.tag_decode.metrics MAX_NO_CALLS=6 CONVERT_LOW_QUALITY_TO_NO_CALL=true OUTPUT=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/1234_8.bam'};
  eq_or_diff([split"=",$bsub_command], [split"=",$expected_cmd], 'correct bsub command for lane 8 (with adapter detection)');


  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => q{123456_IL2_1234},
    runfolder_path => $util->analysis_runfolder_path(),
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    id_run => 8033,
    bam_basecall_path => $util->analysis_runfolder_path() . q{/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls},
  }); } q{no croak creating bam_generator object};

  isa_ok($bam_generator, q{npg_pipeline::archive::file::generation::illumina2bam}, q{$bam_generator});
  is($bam_generator->extra_tradis_transposon_read, 1, 'TraDIS set');

  isa_ok($bam_generator->lims, 'st::api::lims', 'cached lims object');
  is($bam_generator->lims->batch_id, 17216, 'batch id from lims object');

  $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'}, 
  };
  
  $alims = $bam_generator->lims->associated_child_lims_ia;
  $bsub_command = $bam_generator->_generate_bsub_commands( $arg_refs , $alims->{$position});

  is( $bam_generator->_get_number_of_plexes_excluding_control($tag_list_file, $alims->{$position}), 
  1, 'correct number of plexes');

  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  $expected_cmd = q{bsub -q test -R 'select[mem>2500] rusage[mem=2500,nfs_12=4]' -M2500000 -w'done(123) && done(321)' -J 'illumina2bam_8033_8_20090709-123456' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/log/illumina2bam_8033_8_20090709-123456.%J.out 'set -o pipefail;/software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/Illumina2bam.jar CREATE_MD5_FILE=true I=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto L=8 B=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls LIBRARY_NAME="5206896" SAMPLE_ALIAS="ERS124385,ERS124386,ERS124387,ERS124388,ERS124389,ERS124390,ERS124391,ERS124392,ERS124393,ERS124394,ERS124395,ERS124396,ERS124397,ERS124398,ERS124399,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124385,ERS124386,ERS124387,ERS124388,ERS124389,ERS124390,ERS124391,ERS124392,ERS124393,ERS124394,ERS124395,ERS124396,ERS124397,ERS124398,ERS124399,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400" STUDY_NAME="mouse PiggyBac sequencing: sites of PiggyBac integration into mouse genome" SEC_BC_SEQ=BC SEC_BC_QUAL=QT BC_SEQ=tr BC_QUAL=tq OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamIndexDecoder.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  BARCODE_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/lane_8.taglist METRICS_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/8033_8.bam.tag_decode.metrics MAX_NO_CALLS=6 CONVERT_LOW_QUALITY_TO_NO_CALL=true OUTPUT=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/8033_8.bam'};
  my @expected = split '=',$expected_cmd;
  my @received = split '=',$bsub_command;
  is_deeply(\@received, \@expected, 'correct bsub command for lane 8');
diag $bam_generator->is_paired_read ? q(paired) : q(not paired);

  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => q{123456_IL2_1234},
    runfolder_path => $util->analysis_runfolder_path(),
    timestamp => q{20090709-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    id_run => 8033,
    is_paired_read => 1,
    bam_basecall_path => $util->analysis_runfolder_path() . q{/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls},
  }); } q{no croak creating bam_generator object (paired read)};
  $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };
  $alims = $bam_generator->lims->associated_child_lims_ia;
  $bsub_command = $bam_generator->_generate_bsub_commands( $arg_refs , $alims->{$position});
  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );
  $expected_cmd = q{bsub -q test -R 'select[mem>2500] rusage[mem=2500,nfs_12=4]' -M2500000 -w'done(123) && done(321)' -J 'illumina2bam_8033_8_20090709-123456' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/log/illumina2bam_8033_8_20090709-123456.%J.out 'set -o pipefail;/software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/Illumina2bam.jar CREATE_MD5_FILE=true I=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto L=8 B=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls LIBRARY_NAME="5206896" SAMPLE_ALIAS="ERS124385,ERS124386,ERS124387,ERS124388,ERS124389,ERS124390,ERS124391,ERS124392,ERS124393,ERS124394,ERS124395,ERS124396,ERS124397,ERS124398,ERS124399,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124385,ERS124386,ERS124387,ERS124388,ERS124389,ERS124390,ERS124391,ERS124392,ERS124393,ERS124394,ERS124395,ERS124396,ERS124397,ERS124398,ERS124399,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400,ERS124400" STUDY_NAME="mouse PiggyBac sequencing: sites of PiggyBac integration into mouse genome" SEC_BC_SEQ=BC SEC_BC_QUAL=QT BC_SEQ=tr BC_QUAL=tq OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamAdapterFinder.jar ADAPTER_LENGTH_TAG=a3 ADAPTER_MATCH_TAG=ah CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamIndexDecoder.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  BARCODE_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/lane_8.taglist METRICS_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/8033_8.bam.tag_decode.metrics MAX_NO_CALLS=6 CONVERT_LOW_QUALITY_TO_NO_CALL=true OUTPUT=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls/8033_8.bam'};
  @expected = split '=',$expected_cmd;
  @received = split '=',$bsub_command;
  is_deeply(\@received, \@expected, 'correct bsub command for lane 8 (with adpater detection)');
diag $bam_generator->is_paired_read ? q(paired) : q(not paired);

## test of special 'jecfoo' read 1 index

  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q{t/data/modified_webservice_cache};
  #local $ENV{SAVE2NPG_WEBSERVICE_CACHE} = 1;

  lives_ok { $bam_generator = npg_pipeline::archive::file::generation::illumina2bam->new({
    run_folder => q{121112_HS20_08797_A_C18TEACXX},
    runfolder_path => $util->analysis_runfolder_path(), 
    timestamp => q{20121112-123456},
    verbose => 0,
    conf_path => $conf_path,
    domain => q{test},
    id_run => 8797,
	extra_tradis_transposon_read => 1,
    bam_basecall_path =>  $util->analysis_runfolder_path(). q{/Data/Intensities/BaseCalls},
  }); } q{no croak creating bam_generator object for run 8797};

  $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'}, 
  };
  
  $alims = $bam_generator->lims->associated_child_lims_ia;
  $position = 8;
  $bsub_command = $bam_generator->_generate_bsub_commands( $arg_refs , $alims->{$position});

  $bsub_command = $util->drop_temp_part_from_paths( $bsub_command );

  $expected_cmd = q{bsub -q test -R 'select[mem>2500] rusage[mem=2500,nfs_12=4]' -M2500000 -w'done(123) && done(321)' -J 'illumina2bam_8797_8_20121112-123456' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/log/illumina2bam_8797_8_20121112-123456.%J.out 'set -o pipefail;/software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/Illumina2bam.jar CREATE_MD5_FILE=true I=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto L=8 B=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/BaseCalls LIBRARY_NAME="6045465" SAMPLE_ALIAS="ERS181250,ERS181251,ERS181252,ERS181253,ERS181254,ERS181255,phiX_for_spiked_buffers" STUDY_NAME="ERP001656: Total RNA was extracted from morpholically abnormal and sibling wild type embryos identified by the Zebrafish Mutation Project (http://www.sanger.ac.uk/Projects/D_rerio/zmp/). The 3prime end of fragmented RNA was pulled down using polyToligos attached to magnetic beads, reverse transcribed, made into Illumina libraries and sequenced using IlluminaHiSeq paired-end sequencing. Protocol: Total RNA was extracted from mouse embryos using Trizol and DNase treated. Chemically fragmented RNA was enriched for the 3prime ends by pulled down using an anchored polyToligo attached to magnetic beads. An RNA oligo comprising part of the Illumina adapter 2 was ligated to the 5prime end of the captured RNA and the RNA was eluted from the beads. Reverse transcription was primed with an anchored polyToligo with part of Illumina adapter 1 at the 5prime end followed by 4 random bases, then an A, C or G base, then one of twelve5 base indexing tags and 14 T bases. An Illumina library with full adapter sequence was produced by 15 cycles of PCR.   This data is part of a pre-publication release. For information on the proper use of pre-publication data shared by the Wellcome Trust Sanger Institute (including details of any publication moratoria), please see http://www.sanger.ac.uk/datasharing/,Illumina Controls: SPIKED_CONTROL" SEC_BC_SEQ=BC SEC_BC_QUAL=QT BC_SEQ=tr BC_QUAL=tq FIRST_INDEX=5 FINAL_INDEX=10 FIRST=11 FINAL=75 FIRST=84 FINAL=158 FIRST_INDEX=1 FINAL_INDEX=4 SEC_BC_SEQ=BR SEC_BC_QUAL=QR  OUTPUT=/dev/stdout COMPRESSION_LEVEL=0 | /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamIndexDecoder.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT I=/dev/stdin  BARCODE_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/lane_8.taglist METRICS_FILE=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/8797_8.bam.tag_decode.metrics MAX_NO_CALLS=6 CONVERT_LOW_QUALITY_TO_NO_CALL=true OUTPUT=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/8797_8.bam'};

  @expected = split '=',$expected_cmd;
  @received = split '=',$bsub_command;
  is_deeply(\@received, \@expected, 'correct bsub command for run 8797 lane 8, special "jecfoo" read1 index');


}

1;
__END__
