# $Id: 20-archive_qc.t 15898 2012-07-17 14:30:05Z js10 $
use strict;
use warnings;
use Test::More tests => 36;
use Test::Exception;
use t::util;

use_ok('npg_pipeline::archive::file::qc');

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];
my $run_folder = $util->default_runfolder();
{
   throws_ok {
    npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      conf_path => $conf_path,
      domain => q{test},
    })
  } qr/Attribute[ ][(]qc_to_run[)][ ]is[ ]required/, q{croak on new as no qc_to_run provided};
}

{
  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{qX_yield},
      timestamp => q{20090709-123456},
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak on new, as required params provided};

  $util->create_analysis({qc_dir => 1});

  my @jids;
  lives_ok { @jids = $aqc->run_qc($arg_refs); } q{no croak $aqc->run_qc()};
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_qX_yield_1234_20090709-123456[1-8]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_qX_yield_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qc --check=qX_yield --id_run=1234 --position=`echo $LSB_JOBINDEX` --qc_in=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --qc_out=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};

  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{qX_yield},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 4,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak on new, as required params provided};

  $util->create_analysis({qc_dir => 1});

  my @jids;
  lives_ok { @jids = $aqc->run_qc($arg_refs) } q{no croak $aqc->run_qc()};
  is(scalar@jids, 1, q{1 job id returned});

  my $bsub_command = $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command($arg_refs) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_qX_yield_1234_20090709-123456[4]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_qX_yield_1234_20090709-123456.%I.%J.out 't/bin/software/solexa/bin/qc --check=qX_yield --id_run=1234 --position=`echo $LSB_JOBINDEX` --qc_in=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --qc_out=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc'};
  is( $bsub_command, $expected_command, q{generated bsub command is correct});
}

{
  my $args = {};
  $args->{qc_dir} = [7];

  $util->create_multiplex_analysis($args);
  $util->copy_tag_file(7);

  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $arg_refs->{lane} = 7;

  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{qX_yield},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
      repository => 't/data/sequence',
    });
  } q{no croak on new for a demultiplexed, as required params provided};

  my @jids;
  lives_ok { @jids = $aqc->run_qc($arg_refs); } q{no croak $aqc->run_qc()};
  is(scalar@jids, 2, q{1 job id returned});
  $util->remove_staging;
}

{
  my $args = {};
  $args->{qc_dir} = [7];

  $util->create_multiplex_analysis($args);
  $util->copy_tag_file(7);

  my $aqc;
  my $arg_refs = {};
  $arg_refs->{required_job_completion}  = q{-w'done(123) && done(321)'};
  $arg_refs->{lane} = 7;

  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{insert_size},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
      repository => 't/data/sequence',
    });
  } q{no croak on new for a demultiplexed, as required params provided};

  my @jids;
  lives_ok { @jids = $aqc->run_qc($arg_refs); } q{no croak $aqc->run_qc()};
  is(scalar     @jids, 2, q{2 job ids returned});

  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{cache},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
      repository => 't/data/sequence',
    });
  } q{no croak on new for a demultiplexed, as required params provided};

  @jids = undef;
  lives_ok { @jids = $aqc->run_qc($arg_refs); } q{no croak $aqc->run_qc()};
  is(scalar @jids, 2, q{2 job ids returned});

  my $cmd = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_cache_1234_20090709-123456[7]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_cache_1234_20090709-123456.%I.%J.out 'generate_cached_fastq --path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_1.fastq  --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_2.fastq --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/1234_`echo $LSB_JOBINDEX`_t.fastq'};
  is( $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command( $arg_refs ) ), $cmd, q{correct lane based cache command} );

  $cmd = q{bsub -q test -R 'rusage[nfs_12=1]' -w'done(123) && done(321)' -J 'qc_cache_1234_20090709-123456[7000,7000,7002,7002,7012,7012]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_cache_1234_20090709-123456.%I.%J.out 'generate_cached_fastq --path /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`  --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`/1234_`echo $LSB_JOBINDEX/1000 | bc`_1#`echo $LSB_JOBINDEX%1000 | bc`.fastq  --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/lane`echo $LSB_JOBINDEX/1000 | bc`/1234_`echo $LSB_JOBINDEX/1000 | bc`_2#`echo $LSB_JOBINDEX%1000 | bc`.fastq'};

  is( $util->drop_temp_part_from_paths( $aqc->_generate_bsub_command( $arg_refs, 1 ) ), $cmd, q{correct demultiplex based cache command} );


  $arg_refs->{spiked_phix_sequence} = q[--species PhiX --sequence_type phix];
  $arg_refs->{lane} = 2;

  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{sequence_error},
      verbose => 1,
      timestamp => q{20090709-123456},
      lane => 2,
      conf_path => $conf_path,
      domain => q{test},
    });
  } 'object created ok';

  @jids = undef;
  lives_ok { @jids = $aqc->run_qc($arg_refs); } q{no croak $aqc->run_qc()};
  is(scalar @jids, 1, q{1 job id returned});

  $cmd = q{bsub -q test -R 'select[type==X86_64,mem>6000] rusage[mem=6000,nfs_12=1]' -M6000000 -w'done(123) && done(321)' -J 'qc_sequence_error_spiked_phix_1234_20090709-123456[2]%64' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc/log/qc_sequence_error_spiked_phix_1234_20090709-123456.%I.%J.out -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references' 't/bin/software/solexa/bin/qc --check=sequence_error --id_run=1234 --position=`echo $LSB_JOBINDEX` --species PhiX --sequence_type phix --qc_in=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive --qc_out=/nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/archive/qc'};

  is( $util->drop_temp_part_from_paths($aqc->_generate_bsub_command( $arg_refs )), $cmd, q{correct lane sequence_error4spiked_phix command} );

  is_deeply( [$aqc->_generate_bsub_command( $arg_refs, 1 )], [], q{no jobs created as lane is not indexed} );

  $arg_refs->{lane} = 7;

  lives_ok {
    $aqc = npg_pipeline::archive::file::qc->new({
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      qc_to_run => q{sequence_error},
      verbose => 0,
      timestamp => q{20090709-123456},
      lane => 7,
      conf_path => $conf_path,
      domain => q{test},
    });
  } 'object created ok';

  @jids = undef;
  lives_ok { @jids = $aqc->run_qc($arg_refs); } q{no croak $aqc->run_qc()};
  is(scalar @jids, 0, q{0 job id returned});
  
  $util->remove_staging;
}

{
  $util->create_analysis();

  my $aqc = npg_pipeline::archive::file::qc->new({
      runfolder_path => $util->analysis_runfolder_path(),
      run_folder => $run_folder,
      conf_path => $conf_path,
      domain => q{test},
      qc_to_run => 'gc_bias',
      repository => 't/data/sequence',
  });
  mkdir $aqc->archive_path;
  mkdir $aqc->qc_path;

  is_deeply( $aqc->_lanes_by_split_nonsplit() , {
   'indexed_lanes' => [],
   'non_indexed_lanes' => [
     '1',
     '2',
     '3',
     '4',
     '5',
     '6',
     '7',
     '8'
   ],
   'non_recalibrated_lanes' => [
     '1',
     '2',
     '3',
     '4',
     '5',
     '6',
     '7',
     '8'
   ],
   'non_split_nc_lanes' => [
     '1',
     '2',
     '3',
     '4',
     '5',
     '6',
     '8'
   ],
   'non_split_nc_mp_lanes' => [],
   'recalibrated_lanes' => [],
   'spiked_phix_lanes' => [
     '2'
   ],
   'split_nc_lanes' => [
     '7'
   ],
   'split_nc_mp_lanes' => []
 }, q{correct set of lane info} );

  $aqc->_lanes_by_split_nonsplit({
    indexed_lanes => [1..6],
  });
  is( $aqc->_can_run( 1 ), 0, q{lane is indexed - no run of gc_bias} );
  is( $aqc->_can_run( 8 ), 1, q{lane is not indexed - run gc_bias} );


  $aqc = npg_pipeline::archive::file::qc->new({
      runfolder_path => $util->analysis_runfolder_path(),
      run_folder => $run_folder,
      conf_path => $conf_path,
      domain => q{test},
      qc_to_run => 'tag_metrics',
  });
  $aqc->_lanes_by_split_nonsplit({
    indexed_lanes => [1..6],
  });
  is( $aqc->_can_run( 1 ), 1, q{lane is indexed - run tag metrics} );
  is( $aqc->_can_run( 8 ), 0, q{lane is not indexed - do not run tag_metrics} );
  is( $aqc->_can_run( 1,1 ), 0, q{lane is indexed, using tag index - do not run tag metrics} );
  is( $aqc->_can_run( 8,1 ), 0, q{lane is not indexed, using tag index - do not run tag metrics} );
}

1;