# $Id: 21-files_present.t 15858 2012-07-10 15:36:19Z mg8 $
use strict;
use warnings;
use English qw{-no_match_vars};
use Test::More tests => 3;
use npg_pipeline::base;
use t::util;

my $util = t::util->new();

my $conf_path = $util->conf_path();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

my $runfolder = $util->default_runfolder();
$util->create_analysis();
my $runfolder_path = $util->analysis_runfolder_path();
my $recalibrated_path = $util->standard_analysis_recalibrated_path();

{
  my $files_present = npg_pipeline::base->new(
      id_run => 1234,
      run_folder => $runfolder,
      recalibrated_path => $recalibrated_path,
      lanes => [1,2,5,7,8],
      conf_path => $conf_path,
      domain => q{test},
    );

  my $command_to_test = q{bin/npg_pipeline_files_present};
  foreach my $lane (1 .. 8) {
    $command_to_test .= qq{ --file $runfolder_path/Latest_Summary/archive/completed_lane_$lane};
  }
  my $expected_command = qq{-E 't/bin/software/solexa/$command_to_test'};

  my $result_command = $files_present->files_present_pre_exec_string();
  is( $result_command, $expected_command, q{preexec command is correct} );

  qx{touch $recalibrated_path/archive/completed_lane_1};
  qx{touch $recalibrated_path/archive/completed_lane_2};
  qx{touch $recalibrated_path/archive/completed_lane_5};

  my $output = qx{$command_to_test};
  ok( $CHILD_ERROR, qq{Return code of $CHILD_ERROR as not all files found} );

  qx{touch $recalibrated_path/archive/completed_lane_3};
  qx{touch $recalibrated_path/archive/completed_lane_4};
  qx{touch $recalibrated_path/archive/completed_lane_6};
  qx{touch $recalibrated_path/archive/completed_lane_7};
  qx{touch $recalibrated_path/archive/completed_lane_8};

  $output = qx{$command_to_test};
  ok( ! $CHILD_ERROR, qq{Return code of $CHILD_ERROR as all files found} );
}

1;