# $Id: 25-analysis-create_lane_tag_file.t 15364 2012-03-21 12:46:00Z js10 $
use strict;
use warnings;
use Test::More tests => 43;
use Test::Exception::LessClever;
use File::Slurp;
use t::util;

my $util = t::util->new();
$util->catch_email();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{TEST_FS_RESOURCE} = q{nfs_12};
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

my $conf_path = $util->conf_path();

BEGIN {
  use_ok(q{npg_pipeline::analysis::create_lane_tag_file});
}

my $runfolder_path = $util->analysis_runfolder_path();

{
  $util->set_rta_staging_analysis_area();
  my $create_lane;
  lives_ok {
    $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{no croak creating new object};
  isa_ok($create_lane, q{npg_pipeline::analysis::create_lane_tag_file}, q{$create_lane});

  lives_ok {
    $create_lane->generate();
  } q{no croak running generate()};
}

{
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = 't/data';
  $util->multiplexed_rta_pre_gerald();
  my $create_lane;
  lives_ok {
    $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      lims => st::api::lims->new(id_run => 1234, batch_id=>5378),
    });
  } q{no croak creating new object};

  $create_lane->run()->batch_id(5378);
  is($create_lane->corrected_tag_part_use_bases(), q{Y76,I8,y76}, q{corrected_tag_part_use_bases returns expected use_bases string});

  throws_ok {
    $create_lane->generate();
  } qr{Lane 1\: no expected tag sequence or index given in sequencescape}, q{croak running generate()};
}

{   
   my $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
    });
   my $tag_to_trim = [qw(actgffc aacgffc aacdffcd)];
   is($create_lane->_tag_common_suffix_length($tag_to_trim), 0, 'tag no common suffix');
   
   $tag_to_trim = [qw(actgffc aacgffc aacdffc)];
   is($create_lane->_tag_common_suffix_length($tag_to_trim), 3, 'tag common suffix length 3');

   $tag_to_trim = [qw(aacdffc aacdffc aacdffc)];
   is($create_lane->_tag_common_suffix_length($tag_to_trim), 7, 'all tags are the same'); 
}

{   
  my $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
    });
   my $tag_to_trim = [qw(actgffce aacgffcf aacdffcd)];
   is_deeply($create_lane->_trim_tag_common_suffix($tag_to_trim),[qw(actgffce aacgffcf aacdffcd)], 'no common suffix');

   $tag_to_trim = [qw(actgffc aacgffc aacdffcd)];
   throws_ok {$create_lane->_trim_tag_common_suffix($tag_to_trim)} qr/The given tags are different in length/, 'different length tags';

   $tag_to_trim = [qw(actgffc aacgffc aacdffc)];
   is_deeply($create_lane->_trim_tag_common_suffix($tag_to_trim), [qw(actg aacg aacd)], 'common suffix length 3');

   $tag_to_trim = [qw(aacdffc aacdffc aacdffc)];
   throws_ok {$create_lane->_trim_tag_common_suffix($tag_to_trim)} qr{All tags are the same}, 'all tags are the same'; 
}

{
  my $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
    });
  my $tag_list = [qw(aaa aaa ccc)];
  throws_ok {
   $create_lane->_check_tag_uniqueness($tag_list);
  } qr{The given tags after trimming are not unique}, q{tags are not unique - throw error};
}

{  
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = 't/data';
  $util->multiplexed_rta_pre_gerald();
  my $create_lane;
  lives_ok {
    $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      lims => st::api::lims->new(id_run => 1234, batch_id=>6532),
    });
  } q{no croak creating new object};

  $create_lane->run()->batch_id( 6532 );

   is(join(q[ ], $create_lane->positions), '1 8', 'positions array - {1 8}');

  my $tag_list_lane1 = {1=> 'TAGCTTGT', 2 => 'CGATGTTT', 3 => 'GCCAATGT',};
  my ($index_list, $tag_seq_list) = $create_lane->_process_tag_list($tag_list_lane1);
  is_deeply($index_list, [1,2,3], 'correct index list');
  is_deeply($tag_seq_list, [qw(TAGCTTG CGATGTT GCCAATG)], 'correct tag list after trimming');
  
  $tag_list_lane1 = {'' => 'TAGCTTGT', 2 => 'CGATGTTT', 3 => 'GCCAATGT',};
  ($index_list, $tag_seq_list) = $create_lane->_process_tag_list( $tag_list_lane1);
  is($index_list, undef, 'no index list available');
  is($tag_seq_list, undef, 'no expected tag sequence available');
  
  $tag_list_lane1 = {1 => undef, 2 => 'CGATGTTT', 3 => 'GCCAATGT',};
  ($index_list, $tag_seq_list) = $create_lane->_process_tag_list($tag_list_lane1);
  is($tag_seq_list, undef, 'no expected tag sequence available');
  
  $tag_list_lane1 = {1=> 'TAGCTTGT', 2 => 'CGATGTTT', 3 => 'GCCAATGT',};
  ($index_list, $tag_seq_list) = $create_lane->_process_tag_list($tag_list_lane1);
  is_deeply($create_lane->_check_tag_length($tag_seq_list),[qw(TAGCTTG CGATGTT GCCAATG)], 'expected tag sequence length are the same as the index_length after trimming common suffix');
  
  $tag_list_lane1 = {1=> 'TAGCTTGTTGA', 2 => 'TGCGATGTTAA', 3 => 'GGCCAATGGGG',};
  ($index_list, $tag_seq_list) = $create_lane->_process_tag_list($tag_list_lane1);
  is_deeply($create_lane->_check_tag_length($tag_seq_list),[qw(TAGCTTGT TGCGATGT GGCCAATG)], 'expected tag sequence length are the same as the index_length after trimming common suffix');
  lives_ok {
    $create_lane->generate();
  } q{no croak running generate()};
}

{
  local $ENV{NPG_WEBSERVICE_CACHE_DIR} = 't/data';
  $util->multiplexed_rta_pre_gerald();
  my $create_lane;
  lives_ok {
    $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      lims => st::api::lims->new(id_run => 1234, batch_id=>6713),
      lanes => [1],
    });
  } q{no croak creating new object};

  $create_lane->run()->batch_id( 6713 );

  lives_ok {
    $create_lane->generate();
  } q{no croak running generate() for batch 6713};

  my $tag_list = $create_lane->runfolder_path() . qq{/lane_1.taglist};
  my $file_contents;
  lives_ok {$file_contents = read_file($tag_list);} 'reading tag list file';
  my $expected = qq[barcode_sequence\tbarcode_name\tlibrary_name\tsample_name\tdescription\nATCACG\t1\t214599\tYeast\tRapid high-resolution QTL mapping in yeast: Sequencing DNA from an entire pool of segregants before and after a selection step to map alleles responsible for increased growth in a restrictive condition.\nCGATGT\t2\t214599\tYeast\tRapid high-resolution QTL mapping in yeast: Sequencing DNA from an entire pool of segregants before and after a selection step to map alleles responsible for increased growth in a restrictive condition.];
  is($file_contents, $expected, 'tag list file contents as expected');

  $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      lims => st::api::lims->new(id_run => 1234, batch_id=>6713),
      lanes => [1,2],
    });
  $create_lane->run()->batch_id( 6713 );
  is(join(q[ ], $create_lane->positions), '1 2', 'positions array - {1 2}');

  throws_ok {$create_lane->generate(); } qr/Position 2 is not available in LIMS/, q{no croak running generate() for batch 6713 without specifying correct lanes};

  $create_lane = npg_pipeline::analysis::create_lane_tag_file->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      runfolder_path => $runfolder_path,
      conf_path => $conf_path,
      domain => q{test},
      lims => st::api::lims->new(id_run => 1234, batch_id=>6713),
    });
  $create_lane->run()->batch_id( 6713 );
  is(join(q[ ], $create_lane->positions), '1', 'positions array - {1}');

  lives_ok {$create_lane->generate(); } q{no croak running generate() for batch 6713};

  my $tags_length_checked;
  $create_lane->_index_length( 5 );
  my $expected_tag_results = [ qw{ AAAAA AAAAA AAAAA AAAAA } ];
  lives_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAA AAAAA AAAAA AAAAA }
    ] );
  } q{tags returned ok};
  is_deeply( $tags_length_checked, $expected_tag_results, q{tags all same size, same as index length} );
  lives_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAAC AAAAAC AAAAAC AAAAAC }
    ] );
  } q{tags returned ok};
  is_deeply( $tags_length_checked, $expected_tag_results, q{tags all same size, longer than index length} );

  $create_lane->_index_length( 6 );
  lives_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAA AAAAA AAAAA AAAAACCCC }
    ] );
  } q{tags returned ok};
  is_deeply( $tags_length_checked, $expected_tag_results, q{one tag longer, and it is longer than index length} );
  $create_lane->_index_length( 12 );
  throws_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAA AAAAA AAAAA AAAAACCCC }
    ] );
  } qr{AAAAA:AAAAA:AAAAA:AAAAACCCC}, q{2 different lengths, longest shorter than the index_length};
  $create_lane->_index_length( 9 );
  throws_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAA AAAAACCCC AAAAA AAAAACCCC }
    ] );
  } qr{AAAAA:AAAAACCCC:AAAAA:AAAAACCCC}, q{2 different lengths, more than one longest};
  $create_lane->_index_length( 9 );
  throws_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAA AAAAA AAAAA AAAA }
    ] );
  } qr{AAAAA:AAAAA:AAAAA:AAAA}, q{2 different lengths, only one shortest};
  throws_ok {
    $tags_length_checked = $create_lane->_check_tag_length( [
      qw{ AAAAAC AAAAA AAAAA AAAA }
    ] );
  } qr{AAAAAC:AAAAA:AAAAA:AAAA}, q{3 different lengths};
}

1;
__END__
