# $Id: 25-analysis-split_bam_by_tag.t 15580 2012-05-14 14:36:16Z gq1 $
use strict;
use warnings;
use Test::More tests => 6;
use Test::Exception;
use t::util;

use_ok('npg_pipeline::analysis::split_bam_by_tag');

my $util = t::util->new();
my $conf_path = $util->conf_path();
my $runfolder_path = $util->analysis_runfolder_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

{ 
  $util->create_multiplex_analysis();
  my $generator;
  lives_ok { $generator = npg_pipeline::analysis::split_bam_by_tag->new({
    run_folder => q{123456_IL2_1234},
    runfolder_path => $runfolder_path,
    timestamp => q{20090709-123456},
    verbose => 1,
    conf_path => $conf_path,
    domain => q{test},
    no_bsub => 1,
  }); } q{no croak creating $fastq_generator object};
  isa_ok($generator, q{npg_pipeline::analysis::split_bam_by_tag}, q{$generator});

  my $arg_refs = {
    position => q{`echo $LSB_JOBINDEX`},
    bam => q{1234_`echo $LSB_JOBINDEX`.bam},
    output_prefix => '/tmp/1234_`echo $LSB_JOBINDEX`',
    required_job_completion => q{-w'done(123) && done(321)'},
    array_string => q{[1-4,8]},
  };

  my $bsub_command = $util->drop_temp_part_from_paths( $generator->_generate_bsub_command( $arg_refs ) );
  my $expected_command = q{bsub -q test -R 'rusage[nfs_12=8]' -w'done(123) && done(321)' -J split_bam_by_tag_1234_20090709-123456[1-4,8] -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/C1-18-Firecrest1.3.4_09-07-2009_auto/Bustard1.3.4_09-07-2009_auto/GERALD_09-07-2009_auto/log/split_bam_by_tag_1234_20090709-123456.%I.%J.out '/software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/SplitBamByReadGroup.jar TRIM=1 CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT  I=1234_`echo $LSB_JOBINDEX`.bam O=/tmp/1234_`echo $LSB_JOBINDEX`'};

  is( $bsub_command, $expected_command, q{generated bsub command is correct} );

  my $runfolder = $generator->runfolder_path();
  `cp t/data/tag_files/lane_7.tag $runfolder`; 
  `cp t/data/tag_files/lane_7.tag $runfolder/lane_1.tag`; 

  my @jids;
  lives_ok { @jids = $generator->generate( $arg_refs ); } q{no croak running generate};
  is(scalar@jids, 1, q{1 jobs ids returned for lane 1 and 7});
}

1;
