# $Id: 25-harold_calibration_bam.t 16492 2013-01-15 17:16:26Z ib5 $
use strict;
use warnings;
use Test::More tests => 28;
use Test::Exception::LessClever;
use t::util;
use Cwd;

my $util = t::util->new();
$util->catch_email();

my $curdir = getcwd();
my $repos = join q[/], $curdir, 't/data/sequence';

$ENV{TEST_BIN} = $curdir . q{/t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
$ENV{TEST_FS_RESOURCE} = q{nfs_12};
$ENV{NPG_WEBSERVICE_CACHE_DIR} = $curdir . q{/t/data};

my $id_run;

BEGIN {
  use_ok(q{npg_pipeline::analysis::harold_calibration_bam});
}

my $conf_path = $util->conf_path();

my $runfolder_path = $util->analysis_runfolder_path();
my $bustard_home   = qq{$runfolder_path/Data/Intensities};
my $bustard_rta    = qq{$bustard_home/Bustard_RTA};
my $gerald_rta     = qq{$bustard_rta/GERALD_RTA};
my $config_path    = qq{$runfolder_path/Config};

sub set_staging_analysis_area {
  `rm -rf /tmp/nfs/sf45`;
  `mkdir -p $bustard_rta`;
  `mkdir $config_path`;
  `cp t/data/Recipes/Recipe_GA2_37Cycle_PE_v6.1.xml $runfolder_path/`;
  `cp t/data/Recipes/TileLayout.xml $config_path/`;
  return 1;
}

{
  set_staging_analysis_area();
  my $harold;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      conf_path => $conf_path,
      domain => q{test},
      log_file_path => $runfolder_path,
      log_file_name => q{npg_pipeline_pb_cal_20091028-101635.log},
      verbose => 1,
      repository => $repos,
    });
  } q{create $harold object ok};

  isa_ok($harold, q{npg_pipeline::analysis::harold_calibration_bam}, q{$harold});

  my @job_ids = $harold->generate_calibration_table({});
  is( scalar @job_ids, 1, q{only lane 2 is spiked phix and 1 job submitted for calibration table creation} );
  is( $harold->find_bustard(), $bustard_rta, q{bustard dir found ok} );

  @job_ids = $harold->generate_recalibrated_bam({});
  is( scalar @job_ids, 8, q{only lane 2 is spiked phix but 8 jobs submitted for recalibration} );
  is( $harold->find_bustard(), $bustard_rta, q{bustard dir found ok} );
}

{
  set_staging_analysis_area();
  my $harold;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => 1234,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 0,
      conf_path => $conf_path,
      domain => q{test},
      repository => $repos,
      dif_files_path   => $bustard_home,
      spatial_filter => 1,
    });
  } q{create $harold object ok};

  my $req_job_completion = q{-w'done(123) && done(321)'};
  my $arg_refs = {
    required_job_completion => $req_job_completion,
  };

  my @job_ids;

  my $job = qq{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository $curdir/t/data/sequence' -o log/PB_cal_table_1234_4_20091028-101635.%J.out -J PB_cal_table_1234_4_20091028-101635 -R 'select[mem>3072] rusage[mem=3072,nfs_12=8] span[hosts=1]' -M 3072000 -w'done(123) && done(321)' " t/bin/software/solexa/bin/pb_calibration/v9.0/pb_calibration --intensity_dir $bustard_home --t_filter 2 --prefix 1234_4 --cstart 1 --bam pb_align_1234_4.bam "};

  lives_ok {
    @job_ids = $harold->generate_calibration_table( $arg_refs );
  } q{no croak submitting calibration table jobs};

  is( scalar @job_ids, 1, q{1 jobs created});

  is( $harold->_calibration_table_bsub_command( {
   dir => $bustard_rta,
   position => 4,
   job_dependencies => $req_job_completion,
   ref_seq => q{phix-illumina.fa},
  } ), $job, q{generated bsub command is correct} );

  my $cal_table = q{1234_4_train_purity_cycle_caltable.txt};
  is( $harold->calibration_table_name( { id_run => 1234, position=>4 } ), $cal_table, q{generated calibration table name is correct});
  $job = qq{bsub -q test -o log/PB_cal_score_1234_3_20091028-101635.%J.out -J PB_cal_score_1234_3_20091028-101635 -R 'select[mem>1725] rusage[mem=1725,nfs_12=8] span[hosts=1]' -M 1725000 -w'done(123) && done(321)' " if [[ -f pb_align_1234_3.bam ]]; then echo phix alignment so merging alignments with 1>&2; set -o pipefail; (if [ -f 1234_4_train_purity_cycle_caltable.txt ]; then echo  recalibrated qvals 1>&2; t/bin/software/solexa/bin/pb_calibration/v9.0/pb_predictor --u --bam ../1234_3.bam --intensity_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities --cstart 1 --ct 1234_4_train_purity_cycle_caltable.txt ; else echo no recalibration 1>&2; cat ../1234_3.bam ; fi;) |  ( t/bin/software/solexa/bin/pb_calibration/v9.0/spatial_filter -u -a -f -F pb_align_1234_3.bam.filter - 2> >( tee /dev/stderr | t/bin/software/solexa/bin/qc --check spatial_filter --id_run 1234 --position 3 --qc_out /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/Bustard_RTA/PB_cal/archive/qc )  ) |  /software/bin/java -Xmx1024m -jar t/bin/software/solexa/bin/aligners/illumina2bam/Illumina2bam-tools-1.00/BamMerger.jar CREATE_MD5_FILE=true VALIDATION_STRINGENCY=SILENT KEEP=true I=/dev/stdin REPLACE_QUAL=true O=1234_3.bam ALIGNED=pb_align_1234_3.bam; else echo symlinking as no phix alignment 1>&2; rm -f 1234_3.bam; ln -s ../1234_3.bam 1234_3.bam; rm -f 1234_3.bam.md5; ln -s ../1234_3.bam.md5 1234_3.bam.md5; fi "};
  my $expect_job = $harold->_recalibration_bsub_command( {
    position => 3,
    job_dependencies => $req_job_completion,
    ct => $cal_table,
  } );

  is($util->drop_temp_part_from_paths( $expect_job ) , $job, q{generated bsub command for recalibration job is correct});

  lives_ok {
    @job_ids = $harold->generate_recalibrated_bam($arg_refs);
  } q{no croak submitting recalibration jobs};
  is( scalar @job_ids, 8, q{8 jobs created});
}

{
  set_staging_analysis_area();
  my $harold;
  $id_run = 4846;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => $id_run,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 0,
      conf_path => $conf_path,
      domain => q{test},
      repository => $repos,
      bam_basecall_path => $runfolder_path . q{/Data/Intensities/BaseCalls},
      no_bsub => 1,
    });
  } q{create $harold object ok};

  isa_ok($harold, q{npg_pipeline::analysis::harold_calibration_bam}, q{$harold});

  my @job_ids = $harold->generate_alignment_files({});
  is( scalar @job_ids, 0, q{no job ids for alignment as no spiked phix lane} );

  @job_ids = $harold->generate_calibration_table({});
  is( scalar @job_ids, 0, q{no job ids for calibration tabke as no spiked phix lane} );

  @job_ids = $harold->generate_recalibrated_bam({});
  is( scalar @job_ids, 8, q{8 job ids for recalibration even if no spiked phix lane} );
}

{
  set_staging_analysis_area();
  my $harold;
  $id_run = 1234;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => $id_run,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 0,
      conf_path => $conf_path,
      domain => q{test},
      repository => $repos,
      dif_files_path => $runfolder_path . q{/Data/Intensities},
      bam_basecall_path => $runfolder_path . q{/Data/Intensities/BaseCalls},
      no_bsub => 1,
      spatial_filter => 1,
    });
  } q{create $harold object ok};

  my $arg_refs = {
    timestamp => q{20091028-101635},
    position => 1,
    job_dependencies => q{-w 'done(1234) && done(4321)'},
    ref_seq => q{t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt},
  };

  my $single_read_alignment_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -o log/PB_cal_align_1234_1_20091028-101635.%J.out -J PB_cal_align_1234_1_20091028-101635 -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M 8000000 -n 6 -w 'done(1234) && done(4321)' ' t/bin/software/solexa/bin/pb_calibration/v9.0/pb_align --aln_parms "-t "`echo $LSB_MCPU_HOSTS | cut -d " " -f2`  --spatial_filter --sf_parms "--region_size 400 --region_min_count 10 --region_mismatch_threshold 20 --region_insertion_threshold 21 --region_deletion_threshold 22 " --intensity_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities --ref t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt --read 1 --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_1.bam --prefix pb_align_1234_1 --pf_filter '};
  my $paired_read_alignment_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -o log/PB_cal_align_1234_1_20091028-101635.%J.out -J PB_cal_align_1234_1_20091028-101635 -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M 8000000 -n 6 -w 'done(1234) && done(4321)' ' t/bin/software/solexa/bin/pb_calibration/v9.0/pb_align --aln_parms "-t "`echo $LSB_MCPU_HOSTS | cut -d " " -f2`  --spatial_filter --sf_parms "--region_size 400 --region_min_count 10 --region_mismatch_threshold 20 --region_insertion_threshold 21 --region_deletion_threshold 22 " --intensity_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities --ref t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt --read1 1 --read2 2 --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_1.bam --prefix pb_align_1234_1 --pf_filter '};
  my $spiked_read_alignment_command = q{bsub -q test -E 't/bin/software/solexa/bin/npg_pipeline_preexec_references --repository t/data/sequence' -o log/PB_cal_align_1234_1_20091028-101635.%J.out -J PB_cal_align_1234_1_20091028-101635 -R 'select[mem>8000] rusage[mem=8000,nfs_12=4] span[hosts=1]' -M 8000000 -n 6 -w 'done(1234) && done(4321)' ' t/bin/software/solexa/bin/pb_calibration/v9.0/pb_align --aln_parms "-t "`echo $LSB_MCPU_HOSTS | cut -d " " -f2`  --spatial_filter --sf_parms "--region_size 400 --region_min_count 10 --region_mismatch_threshold 20 --region_insertion_threshold 21 --region_deletion_threshold 22 " --intensity_dir /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities --ref t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa --read1 1 --read2 2 --bam /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls/1234_1.bam --prefix pb_align_1234_1 --pf_filter '};

  is( $util->drop_temp_part_from_paths( $harold->_alignment_file_bsub_command( $arg_refs ) ), $single_read_alignment_command, q{single read alignment bsub command is correct} );

  $arg_refs->{is_paired} = 1;
  is( $util->drop_temp_part_from_paths( $harold->_alignment_file_bsub_command( $arg_refs ) ), $paired_read_alignment_command, q{paired read alignment bsub command is correct} );

  $arg_refs->{is_spiked_phix} = 1;
  $arg_refs->{ref_seq} = q{t/data/sequence/references/PhiX/default/all/fasta/phix-illumina.fa};
  is( $util->drop_temp_part_from_paths( $harold->_alignment_file_bsub_command( $arg_refs ) ), $spiked_read_alignment_command, q{paired read alignment bsub command is correct} );

  my @job_ids = $harold->generate_alignment_files({});
  is( scalar @job_ids, 1, q{1 job ids, one spiked phix lane} );
}

{
  set_staging_analysis_area();
  my $harold;
  $id_run = 1234;
  lives_ok {
    $harold = npg_pipeline::analysis::harold_calibration_bam->new({
      id_run => $id_run,
      run_folder => q{123456_IL2_1234},
      runfolder_path => $runfolder_path,
      timestamp => q{20091028-101635},
      lsf_queue => q{test},
      verbose => 0,
      conf_path => $conf_path,
      domain => q{test},
      repository => $repos,
      bam_basecall_path => $runfolder_path . q{/Data/Intensities/BAM_basecalls},
      no_bsub => 1,
    });
  } q{create $harold object ok};

  my $arg_refs = {
    timestamp => q{20091028-101635},
    position => 1,
    job_dependencies => q{-w 'done(1234) && done(4321)'},
    ref_seq => q{t/data/sequence/references/Human/default/all/bwa/someref.fa.bwt},
  };

  my $expected_command = q(bsub -q test -o log/basecall_stats_1234_20091028-101635.%J.out -J basecall_stats_1234_20091028-101635 -M 350000 -R 'select[mem>350] rusage[mem=350,nfs_12=4] span[hosts=1]' -n 4  " if [[ -f Makefile ]]; then echo Makefile already present 1>&2; else echo creating bcl2qseq Makefile 1>&2; t/bin/software/solexa/src/OLB-1.9.0a5/bin/setupBclToQseq.py -b /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BaseCalls -o /nfs/sf45/IL2/analysis/123456_IL2_1234/Data/Intensities/BAM_basecalls --overwrite; fi && make -j 4 Matrix Phasing && make -j 4 BustardSummary.x{s,m}l ");
  is( $util->drop_temp_part_from_paths( $harold->_generate_illumina_basecall_stats_command( $arg_refs ) ), $expected_command, q{Illumina basecalls stats generation bsub command is correct} );

  my @job_ids = $harold->generate_illumina_basecall_stats($arg_refs);
  is( scalar @job_ids, 1, q{1 job ids, generate Illumina basecall stats} );

}

1;
