# $Id: 30-run_status-update.t 16101 2012-10-17 09:37:15Z mg8 $
use strict;
use warnings;
use Test::More tests => 9;
use Test::Exception;
use t::util;

my $util = t::util->new();
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

use_ok('npg_pipeline::run::status::update');

my $run_folder = $util->test_run_folder();

{
  my $rsu;
  throws_ok {
    $rsu = npg_pipeline::run::status::update->new({
      run    => 'run object',
      id_run => 1234,
      run_folder => $run_folder,
    });
  } qr/Attribute[ ][(]status_one[)][ ]is[ ]required/, q{missing status_one attribute};
}
{
  my $rsu;
  eval {
    $rsu = npg_pipeline::run::status::update->new({
      status_one => 'run archived',
      id_run     => 1234,
      run_folder => $run_folder,
      conf_path => $conf_path,
    });
  };
  isa_ok($rsu, q{npg_pipeline::run::status::update}, q{$rsu});
}
{

  $util->set_rta_staging_analysis_area();

  my $rsu =  npg_pipeline::run::status::update->new(
      status_one => 'run archived',
      id_run     => 1234,
      run_folder => $run_folder,
      runfolder_path => $util->analysis_runfolder_path(),
      conf_path => $conf_path,
      domain => q{test});
  throws_ok {
    $rsu->status_one('test');
  } qr/Cannot[ ]assign[ ]a[ ]value[ ]to[ ]a[ ]read-only[ ]accessor/, q{cannot assign to a read_only accessor - status_one};
  throws_ok {
    $rsu->run('test');
  } qr/Cannot[ ]assign[ ]a[ ]value[ ]to[ ]a[ ]read-only[ ]accessor/, q{cannot assign to a read_only accessor - run};
  throws_ok {
    $rsu->id_run('4321');
  } qr/Cannot[ ]assign[ ]a[ ]value[ ]to[ ]a[ ]read-only[ ]accessor/, q{cannot assign to a read_only accessor - id_run};
  is($rsu->id_run(), 1234, q{id_run value still correct});

}

{
  $util->set_rta_staging_analysis_area();
  my $rsu;
  eval {
    $rsu = npg_pipeline::run::status::update->new({
      status_one => 'run archived',
      id_run     => 1234,
      run_folder =>$run_folder,
      timestamp  => q{20090709-123456},
      runfolder_path => $util->analysis_runfolder_path(),
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
    });
  };

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->remove_staging();
  my $bsub_command = $util->drop_temp_part_from_paths( $rsu->_generate_bsub_command($arg_refs) );
  my $expected_cmd =  q{bsub -w'done(123) && done(321)' -J update_run_status_1234_run_archived_20090709-123456 -q small -o /nfs/sf45/IL2/outgoing/123456_IL2_1234/update_run_status_1234_run_archived_20090709-123456.out 't/bin/software/solexa/bin/npg_pipeline_update_run_status --id_run 1234 --status_one "run archived"'};

  is($bsub_command, $expected_cmd, q{bsub command ok when updating status to "run archived"});

  $util->set_rta_staging_analysis_area();

  eval {
    $rsu = npg_pipeline::run::status::update->new({
      status_one => 'analysis complete',
      id_run     => 1234,
      run_folder =>$run_folder,
      timestamp  => q{20090709-123456},
      runfolder_path => $util->analysis_runfolder_path(),
      recalibrated_path => $util->standard_analysis_recalibrated_path(),
      verbose => 1,
      conf_path => $conf_path,
      domain => q{test},
      lanes => [1..8],
    });
  };

  $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
  };

  $util->remove_staging();
  $bsub_command = $util->drop_temp_part_from_paths( $rsu->_generate_bsub_command($arg_refs) );
  $expected_cmd =  q{bsub -w'done(123) && done(321)' -J update_run_status_1234_analysis_complete_20090709-123456 -q small -E 't/bin/software/solexa/bin/npg_pipeline_files_present --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_1 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_2 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_3 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_4 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_5 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_6 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_7 --file /nfs/sf45/IL2/analysis/123456_IL2_1234/Latest_Summary/archive/completed_lane_8' -o /nfs/sf45/IL2/analysis/123456_IL2_1234/update_run_status_1234_analysis_complete_20090709-123456.out 't/bin/software/solexa/bin/npg_pipeline_update_run_status --id_run 1234 --status_one "analysis complete"'};

  is($bsub_command, $expected_cmd, q{bsub command ok when updating status to "analysis complete"});
}

$util->remove_staging();

1;