# $Id: 35-archive_file_generation-BamClusterCounts.t 15025 2012-01-25 16:46:54Z mg8 $

use strict;
use warnings;
use English qw{-no_match_vars};
use Test::More tests => 22;
use Test::Exception::LessClever;
use t::util;

BEGIN {
  use_ok( q{npg_pipeline::archive::file::BamClusterCounts} );
}

my $util = t::util->new({});
my $conf_path = $util->conf_path();

$ENV{TEST_BIN} = q{t/bin/};
$ENV{TEST_DIR} = $util->temp_directory();
local $ENV{NPG_WEBSERVICE_CACHE_DIR} = q[t/data];

$util->create_multiplex_analysis();
my $analysis_runfolder_path = $util->analysis_runfolder_path();
my $bam_basecall_path = $util->standard_analysis_bustard_path();
qx{cp t/data/summary_files/BustardSummary_mp.xml $bam_basecall_path/BustardSummary.xml};
my $recalibrated_path = $util->standard_analysis_recalibrated_path();
my $archive_path = $recalibrated_path . q{/archive};

{
  my $object;
  lives_ok {
    $object = npg_pipeline::archive::file::BamClusterCounts->new({
      run_folder => q{123456_IL2_1234},
      runfolder_path => $analysis_runfolder_path,
      bam_basecall_path => $bam_basecall_path,
      id_run => 1234,
      timestamp => q{20100907-142417},
      conf_path => $conf_path,
      domain => q{test},
      no_bsub => 1,
    });
  } q{obtain object ok};

  isa_ok( $object, q{npg_pipeline::archive::file::BamClusterCounts}, q{$object} );

  my $arg_refs = {
    required_job_completion => q{-w'done(123) && done(321)'},
    array_string => q{[1-8]},
  };

  my $bsub_command = $util->drop_temp_part_from_paths( qq{bsub -q test -w'done(123) && done(321)' -J 'npg_pipeline_check_bam_file_cluster_count_1234_20100907-142417[1-8]' -o $archive_path/log/npg_pipeline_check_bam_file_cluster_count_1234_20100907-142417.} . q{%I.%J.out 't/bin/software/solexa/bin/npg_pipeline_check_bam_file_cluster_count --id_run=1234 --position=`echo $LSB_JOBINDEX` --archive_path=} . qq{$archive_path --bam_basecall_path=$bam_basecall_path'} );
  is( $util->drop_temp_part_from_paths( $object->_generate_bsub_command( $arg_refs ) ), $bsub_command, q{generated bsub command is correct} );

  my @jids = $object->launch( $arg_refs );
  is( scalar @jids, 1, q{1 job id returned} );
}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::BamClusterCounts->new({
      id_run => 1234,
      position => 5,
      runfolder_path => $analysis_runfolder_path,
      bam_basecall_path => $bam_basecall_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  throws_ok {
    $object->run_cluster_count_check();
  } qr{inconsistent[ ]raw[ ]counts[ ]between[ ]reads}, q{$object->run() throws ok - inconsistent raw};

}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::BamClusterCounts->new({
      id_run => 1234,
      position => 6,
      runfolder_path => $analysis_runfolder_path,
      bam_basecall_path => $bam_basecall_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  throws_ok {
    $object->run_cluster_count_check();
  } qr{inconsistent[ ]pf[ ]counts[ ]between[ ]reads}, q{$object->run() throws ok - inconsistent pf};

}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::BamClusterCounts->new({
      id_run => 1234,
      position => 1,
      runfolder_path => $analysis_runfolder_path,
      bam_basecall_path => $bam_basecall_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};

  qx{mkdir $archive_path/qc};
  qx{cp t/data/bam_flagstats/1234_1_bam_flagstats.json $archive_path/qc };

  is( $object->_bustard_pf_cluster_count(),  15, q{correct pf_cluster_count obtained from BustardSummary.xml}  );
  is( $object->_bustard_raw_cluster_count(), 30, q{correct raw_cluster_count obtained from BustardSummary.xml} );

  lives_ok {
    $object->run_cluster_count_check();
  } qr{check returns ok};

}

{
  my $object;
  lives_ok{
    $object = npg_pipeline::archive::file::BamClusterCounts->new({
      id_run => 1234,
      runfolder_path => $analysis_runfolder_path,
      position => 3,
      bam_basecall_path => $bam_basecall_path,
      archive_path => $archive_path,
      conf_path => $conf_path,
      domain => q{test},
    });
  } q{obtain object ok};
  
  ok( !$object->_bam_cluster_count_total({}), 'no bam cluster count total returned');
  
  qx{mkdir $archive_path/qc};
  qx{cp t/data/bam_flagstats/1234_3_bam_flagstats.json $archive_path/qc };
    
  is( $object->_bam_cluster_count_total({}), 16, 'correct bam cluster count total');
  throws_ok {$object->_check_bam_cluster_count()}  qr{Cluster count in bam files not as expected}, 'Cluster count in bam files not as expected';
  qx{cp t/data/bam_flagstats/1234_3_phix_bam_flagstats.json $archive_path//qc/1234_3_phix_bam_flagstats.json};
  ok($object->_check_bam_cluster_count(), 'Cluster count in bam files as expected');

  my $is_indexed = 1;
  qx{mkdir -p $archive_path/lane3/qc};
  qx{cp t/data/bam_flagstats/1234_3_bam_flagstats.json $archive_path/lane3/qc/1234_3#0_bam_flagstats.json};
  qx{cp t/data/bam_flagstats/1234_3_bam_flagstats.json $archive_path/lane3/qc/1234_3#1_bam_flagstats.json};
  
  is( $object->_bam_cluster_count_total( {plex=>$is_indexed} ), 32, 'correct bam cluster count total for plexes');

  qx{cp t/data/bam_flagstats/1234_3_phix_bam_flagstats.json $archive_path/lane3/qc/1234_3#0_phix_bam_flagstats.json};
  
  is( $object->_bam_cluster_count_total( {plex=>$is_indexed} ), 46, 'correct bam cluster count total for plexes');

}

{
  lives_ok {
    qx{perl bin/npg_pipeline_check_bam_file_cluster_count --conf_path $conf_path --domain live --id_run 1234 --position 3 --bam_basecall_path $bam_basecall_path --archive_path $archive_path}
  } q{script runs ok};
  ok( ! $CHILD_ERROR, q{script completed ok - no croak} );
}

1;
